@@ -7,7 +7,7 @@ use gleam_core::{
     build::{Built, Codegen, Compile, Mode, NullTelemetry, Options, Runtime, Target, Telemetry},
     config::{DenoFlag, PackageConfig},
     error::Error,
-    io::{CommandExecutor, Stdio},
+    io::{Command, CommandExecutor, Stdio},
     paths::ProjectPaths,
     type_::ModuleFunction,
 };
@@ -22,6 +22,7 @@ pub enum Which {
 
 // TODO: test
 pub fn command(
+    paths: &ProjectPaths,
     arguments: Vec<String>,
     target: Option<Target>,
     runtime: Option<Runtime>,
@@ -29,8 +30,30 @@ pub fn command(
     which: Which,
     no_print_progress: bool,
 ) -> Result<(), Error> {
-    let paths = crate::find_project_paths()?;
+    // Don't exit on ctrl+c as it is used by child erlang shell
+    ctrlc::set_handler(move || {}).expect("Error setting Ctrl-C handler");
+    let command = setup(
+        paths,
+        arguments,
+        target,
+        runtime,
+        module,
+        which,
+        no_print_progress,
+    )?;
+    let status = ProjectIO::new().exec(command)?;
+    std::process::exit(status);
+}
 
+pub fn setup(
+    paths: &ProjectPaths,
+    arguments: Vec<String>,
+    target: Option<Target>,
+    runtime: Option<Runtime>,
+    module: Option<String>,
+    which: Which,
+    no_print_progress: bool,
+) -> Result<Command, Error> {
     // Validate the module path
     if let Some(mod_path) = &module {
         if !is_gleam_module(mod_path) {
@@ -48,9 +71,9 @@ pub fn command(
 
     // Download dependencies
     let manifest = if no_print_progress {
-        crate::build::download_dependencies(NullTelemetry)?
+        crate::build::download_dependencies(paths, NullTelemetry)?
     } else {
-        crate::build::download_dependencies(crate::cli::Reporter::new())?
+        crate::build::download_dependencies(paths, crate::cli::Reporter::new())?
     };
 
     // Get the config for the module that is being run to check the target.
@@ -58,13 +81,13 @@ pub fn command(
     // belongs to a dependency or to the root package.
     let (mod_config, package_kind) = match &module {
         Some(mod_path) => {
-            crate::config::find_package_config_for_module(mod_path, &manifest, &paths)?
+            crate::config::find_package_config_for_module(mod_path, &manifest, paths)?
         }
-        _ => (crate::config::root_config()?, PackageKind::Root),
+        _ => (crate::config::root_config(paths)?, PackageKind::Root),
     };
 
     // The root config is required to run the project.
-    let root_config = crate::config::root_config()?;
+    let root_config = crate::config::root_config(paths)?;
 
     // Determine which module to run
     let module = module.unwrap_or(match which {
@@ -97,49 +120,46 @@ pub fn command(
         no_print_progress,
     };
 
-    let built = crate::build::main(options, manifest)?;
+    let built = crate::build::main(paths, options, manifest)?;
 
     // A module can not be run if it does not exist or does not have a public main function.
     let main_function = get_or_suggest_main_function(built, &module, target)?;
 
-    // Don't exit on ctrl+c as it is used by child erlang shell
-    ctrlc::set_handler(move || {}).expect("Error setting Ctrl-C handler");
-
     telemetry.running(&format!("{module}.main"));
 
-    // Run the command
-    let status = match target {
+    // Get the command to run the project.
+    match target {
         Target::Erlang => match runtime {
             Some(r) => Err(Error::InvalidRuntime {
                 target: Target::Erlang,
                 invalid_runtime: r,
             }),
-            _ => run_erlang(&paths, &root_config.name, &module, arguments),
+            _ => run_erlang_command(paths, &root_config.name, &module, arguments),
         },
         Target::JavaScript => match runtime.unwrap_or(mod_config.javascript.runtime) {
-            Runtime::Deno => run_javascript_deno(
-                &paths,
+            Runtime::Deno => run_javascript_deno_command(
+                paths,
                 &root_config,
                 &main_function.package,
                 &module,
                 arguments,
             ),
             Runtime::NodeJs => {
-                run_javascript_node(&paths, &main_function.package, &module, arguments)
+                run_javascript_node_command(paths, &main_function.package, &module, arguments)
+            }
+            Runtime::Bun => {
+                run_javascript_bun_command(paths, &main_function.package, &module, arguments)
             }
-            Runtime::Bun => run_javascript_bun(&paths, &main_function.package, &module, arguments),
         },
-    }?;
-
-    std::process::exit(status);
+    }
 }
 
-fn run_erlang(
+fn run_erlang_command(
     paths: &ProjectPaths,
     package: &str,
     module: &str,
     arguments: Vec<String>,
-) -> Result<i32, Error> {
+) -> Result<Command, Error> {
     let mut args = vec![];
 
     // Specify locations of Erlang applications
@@ -165,15 +185,21 @@ fn run_erlang(
         args.push(argument);
     }
 
-    ProjectIO::new().exec("erl", &args, &[], None, Stdio::Inherit)
+    Ok(Command {
+        program: "erl".to_string(),
+        args,
+        env: vec![],
+        cwd: None,
+        stdio: Stdio::Inherit,
+    })
 }
 
-fn run_javascript_bun(
+fn run_javascript_bun_command(
     paths: &ProjectPaths,
     package: &str,
     module: &str,
     arguments: Vec<String>,
-) -> Result<i32, Error> {
+) -> Result<Command, Error> {
     let mut args = vec!["run".to_string()];
     let entry = write_javascript_entrypoint(paths, package, module)?;
 
@@ -183,15 +209,21 @@ fn run_javascript_bun(
         args.push(arg);
     }
 
-    ProjectIO::new().exec("bun", &args, &[], None, Stdio::Inherit)
+    Ok(Command {
+        program: "bun".to_string(),
+        args,
+        env: vec![],
+        cwd: None,
+        stdio: Stdio::Inherit,
+    })
 }
 
-fn run_javascript_node(
+fn run_javascript_node_command(
     paths: &ProjectPaths,
     package: &str,
     module: &str,
     arguments: Vec<String>,
-) -> Result<i32, Error> {
+) -> Result<Command, Error> {
     let mut args = vec![];
     let entry = write_javascript_entrypoint(paths, package, module)?;
 
@@ -201,7 +233,13 @@ fn run_javascript_node(
         args.push(argument);
     }
 
-    ProjectIO::new().exec("node", &args, &[], None, Stdio::Inherit)
+    Ok(Command {
+        program: "node".to_string(),
+        args,
+        env: vec![],
+        cwd: None,
+        stdio: Stdio::Inherit,
+    })
 }
 
 fn write_javascript_entrypoint(
@@ -222,13 +260,13 @@ main();
     Ok(path)
 }
 
-fn run_javascript_deno(
+fn run_javascript_deno_command(
     paths: &ProjectPaths,
     config: &PackageConfig,
     package: &str,
     module: &str,
     arguments: Vec<String>,
-) -> Result<i32, Error> {
+) -> Result<Command, Error> {
     let mut args = vec![];
 
     // Run the main function.
@@ -295,7 +333,13 @@ fn run_javascript_deno(
         args.push(argument);
     }
 
-    ProjectIO::new().exec("deno", &args, &[], None, Stdio::Inherit)
+    Ok(Command {
+        program: "deno".to_string(),
+        args,
+        env: vec![],
+        cwd: None,
+        stdio: Stdio::Inherit,
+    })
 }
 
 fn add_deno_flag(args: &mut Vec<String>, flag: &str, flags: &DenoFlag) {
