@@ -58,6 +58,12 @@ impl TypedModule {
             .iter()
             .find_map(|statement| statement.find_node(byte_index))
     }
+
+    pub fn find_statement(&self, byte_index: u32) -> Option<&TypedStatement> {
+        self.definitions
+            .iter()
+            .find_map(|definition| definition.find_statement(byte_index))
+    }
 }
 
 /// The `@target(erlang)` and `@target(javascript)` attributes can be used to
@@ -413,7 +419,9 @@ impl TypeAst {
                     None
                 })
                 .or(Some(Located::Annotation(self.location(), type_))),
-            TypeAst::Constructor(TypeAstConstructor { arguments, .. }) => type_
+            TypeAst::Constructor(TypeAstConstructor {
+                arguments, module, ..
+            }) => type_
                 .constructor_types()
                 .and_then(|arg_types| {
                     if let Some(arg) = arguments
@@ -426,6 +434,17 @@ impl TypeAst {
 
                     None
                 })
+                .or(module.as_ref().and_then(|(name, location)| {
+                    if location.contains(byte_index) {
+                        Some(Located::ModuleName {
+                            location: *location,
+                            name,
+                            layer: Layer::Type,
+                        })
+                    } else {
+                        None
+                    }
+                }))
                 .or(Some(Located::Annotation(self.location(), type_))),
             TypeAst::Tuple(TypeAstTuple { elems, .. }) => type_
                 .tuple_types()
@@ -965,6 +984,23 @@ impl TypedDefinition {
             }
         }
     }
+
+    pub fn find_statement(&self, byte_index: u32) -> Option<&TypedStatement> {
+        match self {
+            Definition::Function(function) => {
+                if !function.full_location().contains(byte_index) {
+                    return None;
+                }
+
+                function
+                    .body
+                    .iter()
+                    .find_map(|statement| statement.find_statement(byte_index))
+            }
+
+            _ => None,
+        }
+    }
 }
 
 impl<A, B, C, E> Definition<A, B, C, E> {
@@ -1264,10 +1300,44 @@ impl CallArg<TypedExpr> {
                 .or_else(|| body.iter().find_map(|s| s.find_node(byte_index))),
             // In all other cases we're happy with the default behaviour.
             //
+            _ => match self.value.find_node(byte_index) {
+                Some(located) => Some(located),
+                _ => {
+                    if self.location.contains(byte_index) && self.label.is_some() {
+                        Some(Located::Label(self.location, self.value.type_()))
+                    } else {
+                        None
+                    }
+                }
+            },
+        }
+    }
+
+    pub fn find_statement(&self, byte_index: u32) -> Option<&TypedStatement> {
+        match (self.implicit, &self.value) {
+            (Some(ImplicitCallArgOrigin::Use), TypedExpr::Invalid { .. }) => None,
+            (Some(ImplicitCallArgOrigin::Use), TypedExpr::Fn { body, .. }) => {
+                body.iter().find_map(|s| s.find_statement(byte_index))
+            }
+
+            _ => self.value.find_statement(byte_index),
+        }
+    }
+
+    pub fn is_capture_hole(&self) -> bool {
+        match &self.value {
+            TypedExpr::Var { name, .. } => name == CAPTURE_VARIABLE,
+            _ => false,
+        }
+    }
+}
+
+impl CallArg<TypedPattern> {
+    pub fn find_node(&self, byte_index: u32) -> Option<Located<'_>> {
+        match self.value.find_node(byte_index) {
+            Some(located) => Some(located),
             _ => {
-                if let Some(located) = self.value.find_node(byte_index) {
-                    Some(located)
-                } else if self.location.contains(byte_index) && self.label.is_some() {
+                if self.location.contains(byte_index) && self.label.is_some() {
                     Some(Located::Label(self.location, self.value.type_()))
                 } else {
                     None
@@ -1277,22 +1347,10 @@ impl CallArg<TypedExpr> {
     }
 }
 
-impl CallArg<TypedPattern> {
-    pub fn find_node(&self, byte_index: u32) -> Option<Located<'_>> {
-        if let Some(located) = self.value.find_node(byte_index) {
-            Some(located)
-        } else if self.location.contains(byte_index) && self.label.is_some() {
-            Some(Located::Label(self.location, self.value.type_()))
-        } else {
-            None
-        }
-    }
-}
-
 impl CallArg<UntypedExpr> {
     pub fn is_capture_hole(&self) -> bool {
         match &self.value {
-            UntypedExpr::Var { ref name, .. } => name == CAPTURE_VARIABLE,
+            UntypedExpr::Var { name, .. } => name == CAPTURE_VARIABLE,
             _ => false,
         }
     }
@@ -1524,6 +1582,7 @@ pub enum ClauseGuard<Type, RecordTag> {
         location: SrcSpan,
         type_: Type,
         name: EcoString,
+        definition_location: SrcSpan,
     },
 
     TupleIndex {
@@ -1689,8 +1748,8 @@ impl SrcSpan {
 }
 
 #[derive(Debug, PartialEq, Eq, Clone)]
-pub struct DefinitionLocation<'module> {
-    pub module: Option<&'module str>,
+pub struct DefinitionLocation {
+    pub module: Option<EcoString>,
     pub span: SrcSpan,
 }
 
@@ -1870,7 +1929,7 @@ impl<A> Pattern<A> {
 }
 
 impl TypedPattern {
-    pub fn definition_location(&self) -> Option<DefinitionLocation<'_>> {
+    pub fn definition_location(&self) -> Option<DefinitionLocation> {
         match self {
             Pattern::Int { .. }
             | Pattern::Float { .. }
@@ -1950,7 +2009,6 @@ impl TypedPattern {
             | Pattern::VarUsage { .. }
             | Pattern::Assign { .. }
             | Pattern::Discard { .. }
-            | Pattern::BitArray { .. }
             | Pattern::StringPrefix { .. }
             | Pattern::Invalid { .. } => Some(Located::Pattern(self)),
 
@@ -1972,6 +2030,11 @@ impl TypedPattern {
                 .or_else(|| tail.as_ref().and_then(|p| p.find_node(byte_index))),
 
             Pattern::Tuple { elems, .. } => elems.iter().find_map(|p| p.find_node(byte_index)),
+
+            Pattern::BitArray { segments, .. } => segments
+                .iter()
+                .find_map(|segment| segment.find_node(byte_index))
+                .or(Some(Located::Pattern(self))),
         }
         .or(Some(Located::Pattern(self)))
     }
@@ -2048,6 +2111,16 @@ impl TypedExprBitArraySegment {
     }
 }
 
+impl TypedPatternBitArraySegment {
+    pub fn find_node(&self, byte_index: u32) -> Option<Located<'_>> {
+        self.value.find_node(byte_index).or_else(|| {
+            self.options
+                .iter()
+                .find_map(|option| option.find_node(byte_index))
+        })
+    }
+}
+
 pub type TypedConstantBitArraySegmentOption = BitArrayOption<TypedConstant>;
 
 #[derive(Debug, PartialEq, Eq, Clone)]
@@ -2177,10 +2250,34 @@ impl<A> BitArrayOption<A> {
     }
 }
 
+impl BitArrayOption<TypedPattern> {
+    pub fn find_node(&self, byte_index: u32) -> Option<Located<'_>> {
+        match self {
+            BitArrayOption::Bytes { .. }
+            | BitArrayOption::Int { .. }
+            | BitArrayOption::Float { .. }
+            | BitArrayOption::Bits { .. }
+            | BitArrayOption::Utf8 { .. }
+            | BitArrayOption::Utf16 { .. }
+            | BitArrayOption::Utf32 { .. }
+            | BitArrayOption::Utf8Codepoint { .. }
+            | BitArrayOption::Utf16Codepoint { .. }
+            | BitArrayOption::Utf32Codepoint { .. }
+            | BitArrayOption::Signed { .. }
+            | BitArrayOption::Unsigned { .. }
+            | BitArrayOption::Big { .. }
+            | BitArrayOption::Little { .. }
+            | BitArrayOption::Native { .. }
+            | BitArrayOption::Unit { .. } => None,
+            BitArrayOption::Size { value, .. } => value.find_node(byte_index),
+        }
+    }
+}
+
 #[derive(Debug, Clone, Copy, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
 pub enum TodoKind {
     Keyword,
-    EmptyFunction,
+    EmptyFunction { function_location: SrcSpan },
     IncompleteUse,
     EmptyBlock,
 }
@@ -2377,13 +2474,6 @@ impl TypedStatement {
         }
     }
 
-    pub fn is_non_pipe_expression(&self) -> bool {
-        match self {
-            Statement::Expression(expression) => !expression.is_pipeline(),
-            _ => false,
-        }
-    }
-
     pub fn location(&self) -> SrcSpan {
         match self {
             Statement::Expression(expression) => expression.location(),
@@ -2411,7 +2501,7 @@ impl TypedStatement {
         }
     }
 
-    pub fn definition_location(&self) -> Option<DefinitionLocation<'_>> {
+    pub fn definition_location(&self) -> Option<DefinitionLocation> {
         match self {
             Statement::Expression(expression) => expression.definition_location(),
             Statement::Assignment(_) => None,
@@ -2433,6 +2523,22 @@ impl TypedStatement {
         }
     }
 
+    pub fn find_statement(&self, byte_index: u32) -> Option<&TypedStatement> {
+        match self {
+            Statement::Use(use_) => use_.call.find_statement(byte_index),
+            Statement::Expression(expression) => expression.find_statement(byte_index),
+            Statement::Assignment(assignment) => {
+                assignment.value.find_statement(byte_index).or_else(|| {
+                    if assignment.location.contains(byte_index) {
+                        Some(self)
+                    } else {
+                        None
+                    }
+                })
+            }
+        }
+    }
+
     pub fn type_defining_location(&self) -> SrcSpan {
         match self {
             Statement::Expression(expression) => expression.type_defining_location(),
@@ -2482,3 +2588,71 @@ impl TypedAssignment {
         self.value.type_()
     }
 }
+
+/// A pipeline is desugared to a series of assignments:
+///
+/// ```gleam
+/// wibble |> wobble |> woo
+/// ```
+///
+/// Becomes:
+///
+/// ```erl
+/// Pipe1 = wibble
+/// Pipe2 = wobble(Pipe1)
+/// woo(Pipe2)
+/// ```
+///
+/// This represents one of such assignments once the pipeline has been desugared
+/// and each step has been typed.
+///
+/// > We're not using a more general `TypedAssignment` node since that has much
+/// > more informations to carry around. This one is limited since we know it
+/// > will always be in the form `VarName = <Expr>`, with no patterns on the
+/// > left hand side of the assignment.
+/// > Being more constrained simplifies code generation for pipelines!
+///
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct TypedPipelineAssignment {
+    pub location: SrcSpan,
+    pub name: EcoString,
+    pub value: Box<TypedExpr>,
+}
+
+impl TypedPipelineAssignment {
+    pub fn find_node(&self, byte_index: u32) -> Option<Located<'_>> {
+        self.value.find_node(byte_index)
+    }
+
+    pub fn find_statement(&self, byte_index: u32) -> Option<&TypedStatement> {
+        self.value.find_statement(byte_index)
+    }
+
+    pub fn type_(&self) -> Arc<Type> {
+        self.value.type_()
+    }
+}
+
+/// The kind of desugaring that might take place when rewriting a pipeline to
+/// regular assignments.
+///
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum PipelineAssignmentKind {
+    /// In case `a |> b(c)` is desugared to `b(a, c)`.
+    FirstArgument {
+        /// The location of the second argument of the call, in case there's any:
+        /// - `a |> b(c, d)`: here it's `Some` wrapping the location of `c`.
+        /// - `a |> b()`: here it's `None`.
+        second_argument: Option<SrcSpan>,
+    },
+
+    /// In case there's an explicit hole and `a |> b(_, c)` is desugared to
+    /// `b(a, c)`.
+    Hole { hole: SrcSpan },
+
+    /// In case `a |> b(c)` is desugared to `b(c)(a)`
+    FunctionCall,
+
+    /// In case there's an echo in the middle of a pipeline `a |> echo`
+    Echo,
+}
