@@ -319,6 +319,11 @@ pub trait UntypedExprFolder: TypeAstFolder + UntypedConstantFolder + PatternFold
                 message,
             } => self.fold_todo(kind, location, message),
 
+            UntypedExpr::Echo {
+                location,
+                expression,
+            } => self.fold_echo(location, expression),
+
             UntypedExpr::Panic { location, message } => self.fold_panic(location, message),
 
             UntypedExpr::BitArray { location, segments } => self.fold_bit_array(location, segments),
@@ -364,6 +369,14 @@ pub trait UntypedExprFolder: TypeAstFolder + UntypedConstantFolder + PatternFold
                 message: message.map(|msg_expr| Box::new(self.fold_expr(*msg_expr))),
             },
 
+            UntypedExpr::Echo {
+                location,
+                expression,
+            } => UntypedExpr::Echo {
+                location,
+                expression: expression.map(|expression| Box::new(self.fold_expr(*expression))),
+            },
+
             UntypedExpr::Block {
                 location,
                 statements,
@@ -457,23 +470,25 @@ pub trait UntypedExprFolder: TypeAstFolder + UntypedConstantFolder + PatternFold
                 clauses,
             } => {
                 let subjects = subjects.into_iter().map(|e| self.fold_expr(e)).collect();
-                let clauses = clauses
-                    .into_iter()
-                    .map(|mut c| {
-                        c.pattern = c
-                            .pattern
-                            .into_iter()
-                            .map(|p| self.fold_pattern(p))
-                            .collect();
-                        c.alternative_patterns = c
-                            .alternative_patterns
-                            .into_iter()
-                            .map(|p| p.into_iter().map(|p| self.fold_pattern(p)).collect())
-                            .collect();
-                        c.then = self.fold_expr(c.then);
-                        c
-                    })
-                    .collect();
+                let clauses = clauses.map(|clauses| {
+                    clauses
+                        .into_iter()
+                        .map(|mut c| {
+                            c.pattern = c
+                                .pattern
+                                .into_iter()
+                                .map(|p| self.fold_pattern(p))
+                                .collect();
+                            c.alternative_patterns = c
+                                .alternative_patterns
+                                .into_iter()
+                                .map(|p| p.into_iter().map(|p| self.fold_pattern(p)).collect())
+                                .collect();
+                            c.then = self.fold_expr(c.then);
+                            c
+                        })
+                        .collect()
+                });
                 UntypedExpr::Case {
                     location,
                     subjects,
@@ -742,7 +757,7 @@ pub trait UntypedExprFolder: TypeAstFolder + UntypedConstantFolder + PatternFold
         &mut self,
         location: SrcSpan,
         subjects: Vec<UntypedExpr>,
-        clauses: Vec<UntypedClause>,
+        clauses: Option<Vec<UntypedClause>>,
     ) -> UntypedExpr {
         UntypedExpr::Case {
             location,
@@ -796,6 +811,17 @@ pub trait UntypedExprFolder: TypeAstFolder + UntypedConstantFolder + PatternFold
         }
     }
 
+    fn fold_echo(
+        &mut self,
+        location: SrcSpan,
+        expression: Option<Box<UntypedExpr>>,
+    ) -> UntypedExpr {
+        UntypedExpr::Echo {
+            location,
+            expression,
+        }
+    }
+
     fn fold_panic(&mut self, location: SrcSpan, message: Option<Box<UntypedExpr>>) -> UntypedExpr {
         UntypedExpr::Panic { location, message }
     }
