@@ -424,14 +424,7 @@ impl io::Write for InMemoryFile {
 }
 
 impl CommandExecutor for InMemoryFileSystem {
-    fn exec(
-        &self,
-        _program: &str,
-        _args: &[String],
-        _env: &[(&str, String)],
-        _cwd: Option<&Utf8Path>,
-        _stdio: Stdio,
-    ) -> Result<i32, Error> {
+    fn exec(&self, _command: Command) -> Result<i32, Error> {
         Ok(0) // Always succeed.
     }
 }
@@ -443,8 +436,8 @@ impl BeamCompiler for InMemoryFileSystem {
         _lib: &Utf8Path,
         _modules: &HashSet<Utf8PathBuf>,
         _stdio: Stdio,
-    ) -> Result<(), Error> {
-        Ok(()) // Always succeed.
+    ) -> Result<Vec<String>, Error> {
+        Ok(Vec::new()) // Always succeed.
     }
 }
 
