@@ -35,6 +35,8 @@ impl Position {
 #[derive(Debug)]
 pub(crate) struct Generator<'module> {
     module_name: EcoString,
+    src_path: &'module Utf8Path,
+    project_root: &'module Utf8Path,
     line_numbers: &'module LineNumbers,
     function_name: Option<EcoString>,
     function_arguments: Vec<Option<&'module EcoString>>,
@@ -54,6 +56,8 @@ impl<'module> Generator<'module> {
     #[allow(clippy::too_many_arguments)] // TODO: FIXME
     pub fn new(
         module_name: EcoString,
+        src_path: &'module Utf8Path,
+        project_root: &'module Utf8Path,
         line_numbers: &'module LineNumbers,
         function_name: EcoString,
         function_arguments: Vec<Option<&'module EcoString>>,
@@ -74,6 +78,8 @@ impl<'module> Generator<'module> {
         Self {
             tracker,
             module_name,
+            src_path,
+            project_root,
             line_numbers,
             function_name,
             function_arguments,
@@ -84,19 +90,19 @@ impl<'module> Generator<'module> {
         }
     }
 
-    pub fn local_var<'a>(&mut self, name: &'a EcoString) -> Document<'a> {
+    pub fn local_var(&mut self, name: &EcoString) -> EcoString {
         match self.current_scope_vars.get(name) {
             None => {
                 let _ = self.current_scope_vars.insert(name.clone(), 0);
-                maybe_escape_identifier_doc(name)
+                maybe_escape_identifier(name)
             }
-            Some(0) => maybe_escape_identifier_doc(name),
-            Some(n) if name == "$" => eco_format!("${n}").to_doc(),
-            Some(n) => eco_format!("{name}${n}").to_doc(),
+            Some(0) => maybe_escape_identifier(name),
+            Some(n) if name == "$" => eco_format!("${n}"),
+            Some(n) => eco_format!("{name}${n}"),
         }
     }
 
-    pub fn next_local_var<'a>(&mut self, name: &'a EcoString) -> Document<'a> {
+    pub fn next_local_var(&mut self, name: &EcoString) -> EcoString {
         let next = self.current_scope_vars.get(name).map_or(0, |i| i + 1);
         let _ = self.current_scope_vars.insert(name.clone(), next);
         self.local_var(name)
@@ -117,15 +123,15 @@ impl<'module> Generator<'module> {
 
     fn tail_call_loop<'a>(&mut self, body: Document<'a>, args: &'a [TypedArg]) -> Output<'a> {
         let loop_assignments = concat(args.iter().flat_map(Arg::get_variable_name).map(|name| {
-            let var = maybe_escape_identifier_doc(name);
+            let var = maybe_escape_identifier(name);
             docvec!["let ", var, " = loop$", name, ";", line()]
         }));
-        Ok(docvec!(
+        Ok(docvec![
             "while (true) {",
-            docvec!(line(), loop_assignments, body).nest(INDENT),
+            docvec![line(), loop_assignments, body].nest(INDENT),
             line(),
             "}"
-        ))
+        ])
     }
 
     fn statement<'a>(&mut self, statement: &'a TypedStatement) -> Output<'a> {
@@ -143,17 +149,19 @@ impl<'module> Generator<'module> {
             TypedExpr::Int { value, .. } => Ok(int(value)),
             TypedExpr::Float { value, .. } => Ok(float(value)),
 
-            TypedExpr::List { elements, tail, .. } => self.not_in_tail_position(|gen| match tail {
-                Some(tail) => {
-                    gen.tracker.prepend_used = true;
-                    let tail = gen.wrap_expression(tail)?;
-                    prepend(elements.iter().map(|e| gen.wrap_expression(e)), tail)
-                }
-                None => {
-                    gen.tracker.list_used = true;
-                    list(elements.iter().map(|e| gen.wrap_expression(e)))
-                }
-            }),
+            TypedExpr::List { elements, tail, .. } => {
+                self.not_in_tail_position(|r#gen| match tail {
+                    Some(tail) => {
+                        r#gen.tracker.prepend_used = true;
+                        let tail = r#gen.wrap_expression(tail)?;
+                        prepend(elements.iter().map(|e| r#gen.wrap_expression(e)), tail)
+                    }
+                    None => {
+                        r#gen.tracker.list_used = true;
+                        list(elements.iter().map(|e| r#gen.wrap_expression(e)))
+                    }
+                })
+            }
 
             TypedExpr::Tuple { elems, .. } => self.tuple(elems),
             TypedExpr::TupleIndex { tuple, index, .. } => self.tuple_index(tuple, *index),
@@ -178,10 +186,11 @@ impl<'module> Generator<'module> {
             } => self.variable(name, constructor),
 
             TypedExpr::Pipeline {
+                first_value,
                 assignments,
                 finally,
                 ..
-            } => self.pipeline(assignments.as_slice(), finally),
+            } => self.pipeline(first_value, assignments.as_slice(), finally),
 
             TypedExpr::Block { statements, .. } => self.block(statements),
 
@@ -210,6 +219,19 @@ impl<'module> Generator<'module> {
 
             TypedExpr::NegateInt { value, .. } => self.negate_with("- ", value),
 
+            TypedExpr::Echo {
+                expression,
+                location,
+                ..
+            } => {
+                let expression = expression
+                    .as_ref()
+                    .expect("echo with no expression outside of pipe");
+                let expresion_doc =
+                    self.not_in_tail_position(|this| this.wrap_expression(expression))?;
+                self.echo(expresion_doc, location)
+            }
+
             TypedExpr::Invalid { .. } => {
                 panic!("invalid expressions should not reach code generation")
             }
@@ -222,7 +244,7 @@ impl<'module> Generator<'module> {
     }
 
     fn negate_with<'a>(&mut self, with: &'static str, value: &'a TypedExpr) -> Output<'a> {
-        self.not_in_tail_position(|gen| Ok(docvec!(with, gen.wrap_expression(value)?)))
+        self.not_in_tail_position(|r#gen| Ok(docvec![with, r#gen.wrap_expression(value)?]))
     }
 
     fn bit_array<'a>(&mut self, segments: &'a [TypedExprBitArraySegment]) -> Output<'a> {
@@ -232,7 +254,7 @@ impl<'module> Generator<'module> {
 
         // Collect all the values used in segments.
         let segments_array = array(segments.iter().map(|segment| {
-            let value = self.not_in_tail_position(|gen| gen.wrap_expression(&segment.value))?;
+            let value = self.not_in_tail_position(|r#gen| r#gen.wrap_expression(&segment.value))?;
 
             if segment.type_ == crate::type_::int() || segment.type_ == crate::type_::float() {
                 let details = self.sized_bit_array_segment_details(segment)?;
@@ -240,7 +262,8 @@ impl<'module> Generator<'module> {
                 if segment.type_ == crate::type_::int() {
                     match (details.size_value, segment.value.as_ref()) {
                         (Some(size_value), TypedExpr::Int { int_value, .. })
-                            if size_value <= SAFE_INT_SEGMENT_MAX_SIZE.into() =>
+                            if size_value <= SAFE_INT_SEGMENT_MAX_SIZE.into()
+                                && (&size_value % BigInt::from(8) == BigInt::ZERO) =>
                         {
                             let bytes = bit_array_segment_int_value_to_bytes(
                                 int_value.clone(),
@@ -253,7 +276,7 @@ impl<'module> Generator<'module> {
 
                         (Some(size_value), _) if size_value == 8.into() => Ok(value),
 
-                        (Some(size_value), _) if size_value <= 0.into() => Ok(docvec![]),
+                        (Some(size_value), _) if size_value <= 0.into() => Ok(nil()),
 
                         _ => {
                             self.tracker.sized_integer_segment_used = true;
@@ -295,7 +318,28 @@ impl<'module> Generator<'module> {
                     }
 
                     // Bit arrays
-                    [Opt::Bytes { .. } | Opt::Bits { .. }] => Ok(docvec![value, ".buffer"]),
+                    [Opt::Bits { .. }] => Ok(value),
+
+                    // Bit arrays with explicit size. The explicit size slices the bit array to the
+                    // specified size. A runtime exception is thrown if the size exceeds the number
+                    // of bits in the bit array.
+                    [Opt::Bits { .. }, Opt::Size { value: size, .. }]
+                    | [Opt::Size { value: size, .. }, Opt::Bits { .. }] => match &**size {
+                        TypedExpr::Int { value: size, .. } => {
+                            self.tracker.bit_array_slice_used = true;
+                            Ok(docvec!["bitArraySlice(", value, ", 0, ", size, ")"])
+                        }
+
+                        TypedExpr::Var { name, .. } => {
+                            self.tracker.bit_array_slice_used = true;
+                            Ok(docvec!["bitArraySlice(", value, ", 0, ", name, ")"])
+                        }
+
+                        _ => Err(Error::Unsupported {
+                            feature: "This bit array segment option".into(),
+                            location: segment.location,
+                        }),
+                    },
 
                     // Anything else
                     _ => Err(Error::Unsupported {
@@ -348,18 +392,9 @@ impl<'module> Generator<'module> {
                     _ => None,
                 };
 
-                if let Some(size_value) = size_value.as_ref() {
-                    if *size_value > BigInt::ZERO && size_value % 8 != BigInt::ZERO {
-                        return Err(Error::Unsupported {
-                            feature: "Non byte aligned array".into(),
-                            location: segment.location,
-                        });
-                    }
-                }
-
                 (
                     size_value,
-                    self.not_in_tail_position(|gen| gen.wrap_expression(size))?,
+                    self.not_in_tail_position(|r#gen| r#gen.wrap_expression(size))?,
                 )
             }
             _ => {
@@ -413,8 +448,8 @@ impl<'module> Generator<'module> {
             | TypedExpr::Case { .. }
             | TypedExpr::Pipeline { .. }
             | TypedExpr::RecordUpdate { .. } => self
-                .immediately_invoked_function_expression(expression, |gen, expr| {
-                    gen.expression(expr)
+                .immediately_invoked_function_expression(expression, |r#gen, expr| {
+                    r#gen.expression(expr)
                 }),
             _ => self.expression(expression),
         }
@@ -436,7 +471,7 @@ impl<'module> Generator<'module> {
             // Here the document is a return statement: `return <expr>;`
             document
         } else {
-            docvec!("(", document, ")")
+            docvec!["(", document, ")"]
         })
     }
 
@@ -484,22 +519,62 @@ impl<'module> Generator<'module> {
             }
             ValueConstructorVariant::ModuleFn { .. }
             | ValueConstructorVariant::ModuleConstant { .. }
-            | ValueConstructorVariant::LocalVariable { .. } => Ok(self.local_var(name)),
+            | ValueConstructorVariant::LocalVariable { .. } => Ok(self.local_var(name).to_doc()),
         }
     }
 
     fn pipeline<'a>(
         &mut self,
-        assignments: &'a [TypedAssignment],
+        first_value: &'a TypedPipelineAssignment,
+        assignments: &'a [(TypedPipelineAssignment, PipelineAssignmentKind)],
         finally: &'a TypedExpr,
     ) -> Output<'a> {
         let count = assignments.len();
-        let mut documents = Vec::with_capacity((count + 1) * 2);
-        for assignment in assignments.iter() {
-            documents.push(self.not_in_tail_position(|gen| gen.assignment(assignment))?);
+        let mut documents = Vec::with_capacity((count + 2) * 2);
+
+        let all_assignments = std::iter::once(first_value)
+            .chain(assignments.iter().map(|(assignment, _kind)| assignment));
+
+        let mut latest_local_var: Option<EcoString> = None;
+        for assignment in all_assignments {
+            match assignment.value.as_ref() {
+                // An echo in a pipeline won't result in an assignment, instead it
+                // just prints the previous variable assigned in the pipeline.
+                TypedExpr::Echo {
+                    expression: None,
+                    location,
+                    ..
+                } => documents.push(self.not_in_tail_position(|this| {
+                    let var = latest_local_var
+                        .as_ref()
+                        .expect("echo with no previous step in a pipe");
+                    this.echo(var.to_doc(), location)
+                })?),
+
+                // Otherwise we assign the intermediate pipe value to a variable.
+                _ => {
+                    documents.push(self.not_in_tail_position(|this| {
+                        this.simple_variable_assignment(&assignment.name, &assignment.value)
+                    })?);
+                    latest_local_var = Some(self.local_var(&assignment.name));
+                }
+            }
+
             documents.push(line());
         }
-        documents.push(self.expression(finally)?);
+
+        match finally {
+            TypedExpr::Echo {
+                expression: None,
+                location,
+                ..
+            } => {
+                let var = latest_local_var.expect("echo with no previous step in a pipe");
+                documents.push(self.echo(var.to_doc(), location)?);
+            }
+            _ => documents.push(self.expression(finally)?),
+        }
+
         Ok(documents.to_doc().force_break())
     }
 
@@ -511,7 +586,12 @@ impl<'module> Generator<'module> {
     }
 
     fn block<'a>(&mut self, statements: &'a Vec1<TypedStatement>) -> Output<'a> {
-        if statements.len() == 1 {
+        if self.scope_position.is_tail() {
+            // If the block is in tail position there's no need to wrap it in an
+            // immediately invoked function expression; we can just return its
+            // last expression.
+            self.statements(statements)
+        } else if statements.len() == 1 {
             match statements.first() {
                 Statement::Expression(expression) => self.child_expression(expression),
 
@@ -522,8 +602,8 @@ impl<'module> Generator<'module> {
                 Statement::Use(use_) => self.child_expression(&use_.call),
             }
         } else {
-            self.immediately_invoked_function_expression(statements, |gen, statements| {
-                gen.statements(statements)
+            self.immediately_invoked_function_expression(statements, |r#gen, statements| {
+                r#gen.statements(statements)
             })
         }
     }
@@ -533,7 +613,7 @@ impl<'module> Generator<'module> {
         let mut documents = Vec::with_capacity(count * 3);
         for (i, statement) in statements.iter().enumerate() {
             if i + 1 < count {
-                documents.push(self.not_in_tail_position(|gen| gen.statement(statement))?);
+                documents.push(self.not_in_tail_position(|r#gen| r#gen.statement(statement))?);
                 if requires_semicolon(statement) {
                     documents.push(";".to_doc());
                 }
@@ -549,6 +629,24 @@ impl<'module> Generator<'module> {
         }
     }
 
+    fn simple_variable_assignment<'a>(
+        &mut self,
+        name: &'a EcoString,
+        value: &'a TypedExpr,
+    ) -> Output<'a> {
+        // Subject must be rendered before the variable for variable numbering
+        let subject = self.not_in_tail_position(|r#gen| r#gen.wrap_expression(value))?;
+        let js_name = self.next_local_var(name);
+        let assignment = docvec!["let ", js_name.clone(), " = ", subject, ";"];
+        let assignment = if self.scope_position.is_tail() {
+            docvec![assignment, line(), "return ", js_name, ";"]
+        } else {
+            assignment
+        };
+
+        Ok(assignment.force_break())
+    }
+
     fn assignment<'a>(&mut self, assignment: &'a TypedAssignment) -> Output<'a> {
         let TypedAssignment {
             pattern,
@@ -561,31 +659,13 @@ impl<'module> Generator<'module> {
         // If it is a simple assignment to a variable we can generate a normal
         // JS assignment
         if let TypedPattern::Variable { name, .. } = pattern {
-            // Subject must be rendered before the variable for variable numbering
-            let subject = self.not_in_tail_position(|gen| gen.wrap_expression(value))?;
-            let js_name = self.next_local_var(name);
-            return Ok(if self.scope_position.is_tail() {
-                docvec![
-                    "let ",
-                    js_name.clone(),
-                    " = ",
-                    subject,
-                    ";",
-                    line(),
-                    "return ",
-                    js_name,
-                    ";"
-                ]
-            } else {
-                docvec!["let ", js_name, " = ", subject, ";"]
-            }
-            .force_break());
+            return self.simple_variable_assignment(name, value);
         }
 
         // Otherwise we need to compile the patterns
         let (subject, subject_assignment) = pattern::assign_subject(self, value);
         // Value needs to be rendered before traversing pattern to have correctly incremented variables.
-        let value = self.not_in_tail_position(|gen| gen.wrap_expression(value))?;
+        let value = self.not_in_tail_position(|r#gen| r#gen.wrap_expression(value))?;
         let mut pattern_generator = pattern::Generator::new(self);
         pattern_generator.traverse_pattern(&subject, pattern)?;
         let compiled = pattern_generator.take_compiled();
@@ -607,7 +687,7 @@ impl<'module> Generator<'module> {
         // If there is a subject name given create a variable to hold it for
         // use in patterns
         let doc = match subject_assignment {
-            Some(name) => docvec!("let ", name, " = ", value, ";", line(), compiled),
+            Some(name) => docvec!["let ", name, " = ", value, ";", line(), compiled],
             None => compiled,
         };
 
@@ -623,7 +703,7 @@ impl<'module> Generator<'module> {
             pattern::assign_subjects(self, subject_values)
                 .into_iter()
                 .unzip();
-        let mut gen = pattern::Generator::new(self);
+        let mut r#gen = pattern::Generator::new(self);
 
         let mut doc = nil();
 
@@ -644,9 +724,10 @@ impl<'module> Generator<'module> {
 
             // A clause can have many patterns `pattern, pattern ->...`
             for multipatterns in multipatterns {
-                let scope = gen.expression_generator.current_scope_vars.clone();
-                let mut compiled = gen.generate(&subjects, multipatterns, clause.guard.as_ref())?;
-                let consequence = gen
+                let scope = r#gen.expression_generator.current_scope_vars.clone();
+                let mut compiled =
+                    r#gen.generate(&subjects, multipatterns, clause.guard.as_ref())?;
+                let consequence = r#gen
                     .expression_generator
                     .expression_flattening_blocks(&clause.then)?;
 
@@ -655,14 +736,14 @@ impl<'module> Generator<'module> {
 
                 // Reset the scope now that this clause has finished, causing the
                 // variables to go out of scope.
-                gen.expression_generator.current_scope_vars = scope;
+                r#gen.expression_generator.current_scope_vars = scope;
 
                 // If the pattern assigns any variables we need to render assignments
                 let body = if compiled.has_assignments() {
-                    let assignments = gen
+                    let assignments = r#gen
                         .expression_generator
                         .pattern_take_assignments_doc(&mut compiled);
-                    docvec!(assignments, line(), consequence)
+                    docvec![assignments, line(), consequence]
                 } else {
                     consequence
                 };
@@ -676,14 +757,14 @@ impl<'module> Generator<'module> {
                     // render just the body as the case does nothing
                     // A block is used as it could declare variables still.
                     doc.append("{")
-                        .append(docvec!(line(), body).nest(INDENT))
+                        .append(docvec![line(), body].nest(INDENT))
                         .append(line())
                         .append("}")
                 } else if is_final_clause {
                     // If this is the final clause and there are no checks then we can
                     // render `else` instead of `else if (...)`
                     doc.append(" else {")
-                        .append(docvec!(line(), body).nest(INDENT))
+                        .append(docvec![line(), body].nest(INDENT))
                         .append(line())
                         .append("}")
                 } else {
@@ -693,11 +774,12 @@ impl<'module> Generator<'module> {
                         " else if ("
                     })
                     .append(
-                        gen.expression_generator
+                        r#gen
+                            .expression_generator
                             .pattern_take_checks_doc(&mut compiled, true),
                     )
                     .append(") {")
-                    .append(docvec!(line(), body).nest(INDENT))
+                    .append(docvec![line(), body].nest(INDENT))
                     .append(line())
                     .append("}")
                 };
@@ -711,8 +793,8 @@ impl<'module> Generator<'module> {
             .zip(subject_values)
             .flat_map(|(assignment_name, value)| assignment_name.map(|name| (name, value)))
             .map(|(name, value)| {
-                let value = self.not_in_tail_position(|gen| gen.wrap_expression(value))?;
-                Ok(docvec!("let ", name, " = ", value, ";", line()))
+                let value = self.not_in_tail_position(|r#gen| r#gen.wrap_expression(value))?;
+                Ok(docvec!["let ", name, " = ", value, ";", line()])
             })
             .try_collect()?;
 
@@ -726,7 +808,7 @@ impl<'module> Generator<'module> {
         message: Option<&'a TypedExpr>,
     ) -> Output<'a> {
         let message = match message {
-            Some(m) => self.not_in_tail_position(|gen| gen.expression(m))?,
+            Some(m) => self.not_in_tail_position(|r#gen| r#gen.expression(m))?,
             None => string("Pattern match failed, no pattern matched the value."),
         };
 
@@ -734,15 +816,19 @@ impl<'module> Generator<'module> {
     }
 
     fn tuple<'a>(&mut self, elements: &'a [TypedExpr]) -> Output<'a> {
-        self.not_in_tail_position(|gen| {
-            array(elements.iter().map(|element| gen.wrap_expression(element)))
+        self.not_in_tail_position(|r#gen| {
+            array(
+                elements
+                    .iter()
+                    .map(|element| r#gen.wrap_expression(element)),
+            )
         })
     }
 
     fn call<'a>(&mut self, fun: &'a TypedExpr, arguments: &'a [TypedCallArg]) -> Output<'a> {
         let arguments = arguments
             .iter()
-            .map(|element| self.not_in_tail_position(|gen| gen.wrap_expression(&element.value)))
+            .map(|element| self.not_in_tail_position(|r#gen| r#gen.wrap_expression(&element.value)))
             .try_collect()?;
 
         self.call_with_doc_args(fun, arguments)
@@ -819,11 +905,11 @@ impl<'module> Generator<'module> {
             }
 
             _ => {
-                let fun = self.not_in_tail_position(|gen| {
+                let fun = self.not_in_tail_position(|r#gen| {
                     let is_fn_literal = matches!(fun, TypedExpr::Fn { .. });
-                    let fun = gen.wrap_expression(fun)?;
+                    let fun = r#gen.wrap_expression(fun)?;
                     if is_fn_literal {
-                        Ok(docvec!("(", fun, ")"))
+                        Ok(docvec!["(", fun, ")"])
                     } else {
                         Ok(fun)
                     }
@@ -861,23 +947,23 @@ impl<'module> Generator<'module> {
         self.current_scope_vars = scope;
         std::mem::swap(&mut self.function_name, &mut name);
 
-        Ok(docvec!(
-            docvec!(
+        Ok(docvec![
+            docvec![
                 fun_args(arguments, false),
                 " => {",
                 break_("", " "),
                 result?
-            )
+            ]
             .nest(INDENT)
             .append(break_("", " "))
             .group(),
             "}",
-        ))
+        ])
     }
 
     fn record_access<'a>(&mut self, record: &'a TypedExpr, label: &'a str) -> Output<'a> {
-        self.not_in_tail_position(|gen| {
-            let record = gen.wrap_expression(record)?;
+        self.not_in_tail_position(|r#gen| {
+            let record = r#gen.wrap_expression(record)?;
             Ok(docvec![record, ".", maybe_escape_property_doc(label)])
         })
     }
@@ -889,15 +975,15 @@ impl<'module> Generator<'module> {
         args: &'a [TypedCallArg],
     ) -> Output<'a> {
         Ok(docvec![
-            self.not_in_tail_position(|gen| gen.assignment(record))?,
+            self.not_in_tail_position(|r#gen| r#gen.assignment(record))?,
             line(),
             self.call(constructor, args)?,
         ])
     }
 
     fn tuple_index<'a>(&mut self, tuple: &'a TypedExpr, index: u64) -> Output<'a> {
-        self.not_in_tail_position(|gen| {
-            let tuple = gen.wrap_expression(tuple)?;
+        self.not_in_tail_position(|r#gen| {
+            let tuple = r#gen.wrap_expression(tuple)?;
             Ok(docvec![tuple, eco_format!("[{index}]")])
         })
     }
@@ -929,24 +1015,24 @@ impl<'module> Generator<'module> {
     }
 
     fn div_int<'a>(&mut self, left: &'a TypedExpr, right: &'a TypedExpr) -> Output<'a> {
-        let left = self.not_in_tail_position(|gen| gen.child_expression(left))?;
-        let right = self.not_in_tail_position(|gen| gen.child_expression(right))?;
+        let left = self.not_in_tail_position(|r#gen| r#gen.child_expression(left))?;
+        let right = self.not_in_tail_position(|r#gen| r#gen.child_expression(right))?;
         self.tracker.int_division_used = true;
-        Ok(docvec!("divideInt", wrap_args([left, right])))
+        Ok(docvec!["divideInt", wrap_args([left, right])])
     }
 
     fn remainder_int<'a>(&mut self, left: &'a TypedExpr, right: &'a TypedExpr) -> Output<'a> {
-        let left = self.not_in_tail_position(|gen| gen.child_expression(left))?;
-        let right = self.not_in_tail_position(|gen| gen.child_expression(right))?;
+        let left = self.not_in_tail_position(|r#gen| r#gen.child_expression(left))?;
+        let right = self.not_in_tail_position(|r#gen| r#gen.child_expression(right))?;
         self.tracker.int_remainder_used = true;
-        Ok(docvec!("remainderInt", wrap_args([left, right])))
+        Ok(docvec!["remainderInt", wrap_args([left, right])])
     }
 
     fn div_float<'a>(&mut self, left: &'a TypedExpr, right: &'a TypedExpr) -> Output<'a> {
-        let left = self.not_in_tail_position(|gen| gen.child_expression(left))?;
-        let right = self.not_in_tail_position(|gen| gen.child_expression(right))?;
+        let left = self.not_in_tail_position(|r#gen| r#gen.child_expression(left))?;
+        let right = self.not_in_tail_position(|r#gen| r#gen.child_expression(right))?;
         self.tracker.float_division_used = true;
-        Ok(docvec!("divideFloat", wrap_args([left, right])))
+        Ok(docvec!["divideFloat", wrap_args([left, right])])
     }
 
     fn equal<'a>(
@@ -957,15 +1043,15 @@ impl<'module> Generator<'module> {
     ) -> Output<'a> {
         // If it is a simple scalar type then we can use JS' reference identity
         if is_js_scalar(left.type_()) {
-            let left_doc = self.not_in_tail_position(|gen| gen.child_expression(left))?;
-            let right_doc = self.not_in_tail_position(|gen| gen.child_expression(right))?;
+            let left_doc = self.not_in_tail_position(|r#gen| r#gen.child_expression(left))?;
+            let right_doc = self.not_in_tail_position(|r#gen| r#gen.child_expression(right))?;
             let operator = if should_be_equal { " === " } else { " !== " };
-            return Ok(docvec!(left_doc, operator, right_doc));
+            return Ok(docvec![left_doc, operator, right_doc]);
         }
 
         // Other types must be compared using structural equality
-        let left = self.not_in_tail_position(|gen| gen.wrap_expression(left))?;
-        let right = self.not_in_tail_position(|gen| gen.wrap_expression(right))?;
+        let left = self.not_in_tail_position(|r#gen| r#gen.wrap_expression(left))?;
+        let right = self.not_in_tail_position(|r#gen| r#gen.wrap_expression(right))?;
         Ok(self.prelude_equal_call(should_be_equal, left, right))
     }
 
@@ -984,7 +1070,7 @@ impl<'module> Generator<'module> {
         } else {
             "!isEqual"
         };
-        docvec!(operator, args)
+        docvec![operator, args]
     }
 
     fn print_bin_op<'a>(
@@ -993,14 +1079,14 @@ impl<'module> Generator<'module> {
         right: &'a TypedExpr,
         op: &'a str,
     ) -> Output<'a> {
-        let left = self.not_in_tail_position(|gen| gen.child_expression(left))?;
-        let right = self.not_in_tail_position(|gen| gen.child_expression(right))?;
-        Ok(docvec!(left, " ", op, " ", right))
+        let left = self.not_in_tail_position(|r#gen| r#gen.child_expression(left))?;
+        let right = self.not_in_tail_position(|r#gen| r#gen.child_expression(right))?;
+        Ok(docvec![left, " ", op, " ", right])
     }
 
     fn todo<'a>(&mut self, message: Option<&'a TypedExpr>, location: &'a SrcSpan) -> Output<'a> {
         let message = match message {
-            Some(m) => self.not_in_tail_position(|gen| gen.expression(m))?,
+            Some(m) => self.not_in_tail_position(|r#gen| r#gen.expression(m))?,
             None => string("`todo` expression evaluated. This code has not yet been implemented."),
         };
         let doc = self.throw_error("todo", &message, *location, vec![]);
@@ -1010,7 +1096,7 @@ impl<'module> Generator<'module> {
 
     fn panic<'a>(&mut self, location: &'a SrcSpan, message: Option<&'a TypedExpr>) -> Output<'a> {
         let message = match message {
-            Some(m) => self.not_in_tail_position(|gen| gen.expression(m))?,
+            Some(m) => self.not_in_tail_position(|r#gen| r#gen.expression(m))?,
             None => string("`panic` expression evaluated."),
         };
         let doc = self.throw_error("panic", &message, *location, vec![]);
@@ -1055,12 +1141,12 @@ impl<'module> Generator<'module> {
     fn module_select<'a>(
         &mut self,
         module: &'a str,
-        label: &'a str,
+        label: &'a EcoString,
         constructor: &'a ModuleValueConstructor,
     ) -> Document<'a> {
         match constructor {
             ModuleValueConstructor::Fn { .. } | ModuleValueConstructor::Constant { .. } => {
-                docvec!["$", module, ".", maybe_escape_identifier_doc(label)]
+                docvec!["$", module, ".", maybe_escape_identifier(label)]
             }
 
             ModuleValueConstructor::Record {
@@ -1174,6 +1260,26 @@ impl<'module> Generator<'module> {
         )
         .group()
     }
+
+    fn echo<'a>(&mut self, expression: Document<'a>, location: &'a SrcSpan) -> Output<'a> {
+        self.tracker.echo_used = true;
+
+        let relative_path = self
+            .src_path
+            .strip_prefix(self.project_root)
+            .unwrap_or(self.src_path)
+            .as_str()
+            .replace("\\", "\\\\");
+
+        let relative_path_doc = EcoString::from(relative_path).to_doc();
+
+        let echo_argument = call_arguments(vec![
+            Ok(expression),
+            Ok(relative_path_doc.surround("\"", "\"")),
+            Ok(self.line_numbers.line_number(location.start).to_doc()),
+        ])?;
+        Ok(self.wrap_return(docvec!["echo", echo_argument]))
+    }
 }
 
 pub fn int(value: &str) -> Document<'_> {
@@ -1305,8 +1411,8 @@ pub(crate) fn guard_constant_expression<'a>(
         Constant::Var { name, .. } => Ok(assignments
             .iter()
             .find(|assignment| assignment.name == name)
-            .map(|assignment| assignment.subject.clone().append(assignment.path.clone()))
-            .unwrap_or_else(|| maybe_escape_identifier_doc(name))),
+            .map(|assignment| assignment.subject.clone())
+            .unwrap_or_else(|| maybe_escape_identifier(name).to_doc())),
 
         expression => constant_expression(Context::Function, tracker, expression),
     }
@@ -1421,12 +1527,12 @@ pub(crate) fn constant_expression<'a>(
 
         Constant::Var { name, module, .. } => Ok({
             match module {
-                None => maybe_escape_identifier_doc(name),
+                None => maybe_escape_identifier(name).to_doc(),
                 Some((module, _)) => {
                     // JS keywords can be accessed here, but we must escape anyway
                     // as we escape when exporting such names in the first place,
                     // and the imported name has to match the exported name.
-                    docvec!["$", module, ".", maybe_escape_identifier_doc(name)]
+                    docvec!["$", module, ".", maybe_escape_identifier(name)]
                 }
             }
         }),
@@ -1434,7 +1540,7 @@ pub(crate) fn constant_expression<'a>(
         Constant::StringConcatenation { left, right, .. } => {
             let left = constant_expression(context, tracker, left)?;
             let right = constant_expression(context, tracker, right)?;
-            Ok(docvec!(left, " + ", right))
+            Ok(docvec![left, " + ", right])
         }
 
         Constant::Invalid { .. } => panic!("invalid constants should not reach code generation"),
@@ -1460,7 +1566,8 @@ fn bit_array<'a>(
             if segment.type_ == crate::type_::int() {
                 match (details.size_value, segment.value.as_ref()) {
                     (Some(size_value), Constant::Int { int_value, .. })
-                        if size_value <= SAFE_INT_SEGMENT_MAX_SIZE.into() =>
+                        if size_value <= SAFE_INT_SEGMENT_MAX_SIZE.into()
+                            && (&size_value % BigInt::from(8) == BigInt::ZERO) =>
                     {
                         let bytes = bit_array_segment_int_value_to_bytes(
                             int_value.clone(),
@@ -1473,7 +1580,7 @@ fn bit_array<'a>(
 
                     (Some(size_value), _) if size_value == 8.into() => Ok(value),
 
-                    (Some(size_value), _) if size_value <= 0.into() => Ok(docvec![]),
+                    (Some(size_value), _) if size_value <= 0.into() => Ok(nil()),
 
                     _ => {
                         tracker.sized_integer_segment_used = true;
@@ -1514,8 +1621,24 @@ fn bit_array<'a>(
                     Ok(docvec!["codepointBits(", value, ")"])
                 }
 
-                // Bit strings
-                [Opt::Bits { .. }] => Ok(docvec![value, ".buffer"]),
+                // Bit arrays
+                [Opt::Bits { .. }] => Ok(value),
+
+                // Bit arrays with explicit size. The explicit size slices the bit array to the
+                // specified size. A runtime exception is thrown if the size exceeds the number
+                // of bits in the bit array.
+                [Opt::Bits { .. }, Opt::Size { value: size, .. }]
+                | [Opt::Size { value: size, .. }, Opt::Bits { .. }] => match &**size {
+                    Constant::Int { value: size, .. } => {
+                        tracker.bit_array_slice_used = true;
+                        Ok(docvec!["bitArraySlice(", value, ", 0, ", size, ")"])
+                    }
+
+                    _ => Err(Error::Unsupported {
+                        feature: "This bit array segment option".into(),
+                        location: segment.location,
+                    }),
+                },
 
                 // Anything else
                 _ => Err(Error::Unsupported {
@@ -1578,15 +1701,6 @@ fn sized_bit_array_segment_details<'a>(
                 _ => None,
             };
 
-            if let Some(size_value) = size_value.as_ref() {
-                if *size_value > BigInt::ZERO && size_value % 8 != BigInt::ZERO {
-                    return Err(Error::Unsupported {
-                        feature: "Non byte aligned array".into(),
-                        location: segment.location,
-                    });
-                }
-            }
-
             (size_value, constant_expr_fun(tracker, size)?)
         }
         _ => {
@@ -1701,6 +1815,7 @@ impl TypedExpr {
             | TypedExpr::Case { .. }
             | TypedExpr::Panic { .. }
             | TypedExpr::Block { .. }
+            | TypedExpr::Echo { .. }
             | TypedExpr::Pipeline { .. }
             | TypedExpr::RecordUpdate { .. } => true,
 
@@ -1764,6 +1879,7 @@ fn requires_semicolon(statement: &TypedStatement) -> bool {
             | TypedExpr::Var { .. }
             | TypedExpr::List { .. }
             | TypedExpr::Call { .. }
+            | TypedExpr::Echo { .. }
             | TypedExpr::Float { .. }
             | TypedExpr::String { .. }
             | TypedExpr::BinOp { .. }
@@ -1793,11 +1909,11 @@ fn requires_semicolon(statement: &TypedStatement) -> bool {
 
 /// Wrap a document in an immediately involked function expression
 fn immediately_invoked_function_expression_document(document: Document<'_>) -> Document<'_> {
-    docvec!(
-        docvec!("(() => {", break_("", " "), document).nest(INDENT),
+    docvec![
+        docvec!["(() => {", break_("", " "), document].nest(INDENT),
         break_("", " "),
         "})()",
-    )
+    ]
     .group()
 }
 
@@ -1833,13 +1949,13 @@ fn record_constructor<'a>(
             construct_record(qualifier, name, vars.clone()),
             ";"
         ];
-        docvec!(
-            docvec!(wrap_args(vars), " => {", break_("", " "), body)
+        docvec![
+            docvec![wrap_args(vars), " => {", break_("", " "), body]
                 .nest(INDENT)
                 .append(break_("", " "))
                 .group(),
             "}",
-        )
+        ]
     }
 }
 
