@@ -12,7 +12,7 @@
 //! <https://www.typescriptlang.org/docs/handbook/declaration-files/introduction.html>
 
 use crate::ast::{AssignName, Publicity};
-use crate::type_::{is_prelude_module, PRELUDE_MODULE_NAME};
+use crate::type_::{PRELUDE_MODULE_NAME, is_prelude_module};
 use crate::{
     ast::{
         CustomType, Definition, Function, Import, ModuleConstant, TypeAlias, TypedArg,
@@ -20,14 +20,14 @@ use crate::{
     },
     docvec,
     javascript::JavaScriptCodegenTarget,
-    pretty::{break_, Document, Documentable},
+    pretty::{Document, Documentable, break_},
     type_::{Type, TypeVar},
 };
-use ecow::{eco_format, EcoString};
+use ecow::{EcoString, eco_format};
 use itertools::Itertools;
 use std::{collections::HashMap, ops::Deref, sync::Arc};
 
-use super::{import::Imports, join, line, lines, wrap_args, Output, INDENT};
+use super::{INDENT, Output, import::Imports, join, line, lines, wrap_args};
 
 /// When rendering a type variable to an TypeScript type spec we need all type
 /// variables with the same id to end up with the same name in the generated
@@ -211,7 +211,7 @@ impl<'a> TypeScriptGenerator<'a> {
         }
 
         if imports.is_empty() && statements.is_empty() {
-            Ok(docvec!("export {}", line()))
+            Ok(docvec!["export {}", line()])
         } else if imports.is_empty() {
             statements.push(line());
             Ok(statements.to_doc())
@@ -451,7 +451,7 @@ impl<'a> TypeScriptGenerator<'a> {
         } else {
             let constructors = constructors.iter().map(|x| {
                 name_with_generics(
-                    super::maybe_escape_identifier_doc(&x.name),
+                    super::maybe_escape_identifier(&x.name).to_doc(),
                     x.arguments.iter().map(|a| &a.type_),
                 )
             });
@@ -488,7 +488,7 @@ impl<'a> TypeScriptGenerator<'a> {
             },
             "class ",
             name_with_generics(
-                super::maybe_escape_identifier_doc(&constructor.name),
+                super::maybe_escape_identifier(&constructor.name).to_doc(),
                 constructor.arguments.iter().map(|a| &a.type_)
             ),
             " extends _.CustomType {"
@@ -506,8 +506,9 @@ impl<'a> TypeScriptGenerator<'a> {
                 let name = arg
                     .label
                     .as_ref()
-                    .map(|(_, s)| super::maybe_escape_identifier_doc(s))
-                    .unwrap_or_else(|| eco_format!("argument${i}").to_doc());
+                    .map(|(_, s)| super::maybe_escape_identifier(s))
+                    .unwrap_or_else(|| eco_format!("argument${i}"))
+                    .to_doc();
                 docvec![name, ": ", self.do_print_force_generic_param(&arg.type_)]
             })),
             ";",
@@ -519,8 +520,9 @@ impl<'a> TypeScriptGenerator<'a> {
                     let name = arg
                         .label
                         .as_ref()
-                        .map(|(_, s)| super::maybe_escape_identifier_doc(s))
-                        .unwrap_or_else(|| eco_format!("{i}").to_doc());
+                        .map(|(_, s)| super::maybe_escape_identifier(s))
+                        .unwrap_or_else(|| eco_format!("{i}"))
+                        .to_doc();
                     docvec![
                         name,
                         ": ",
@@ -536,10 +538,10 @@ impl<'a> TypeScriptGenerator<'a> {
         docvec![head, class_body, line(), "}"]
     }
 
-    fn module_constant(&mut self, name: &'a str, value: &'a TypedConstant) -> Output<'a> {
+    fn module_constant(&mut self, name: &'a EcoString, value: &'a TypedConstant) -> Output<'a> {
         Ok(docvec![
             "export const ",
-            super::maybe_escape_identifier_doc(name),
+            super::maybe_escape_identifier(name),
             ": ",
             self.print_type(&value.type_()),
             ";",
@@ -548,7 +550,7 @@ impl<'a> TypeScriptGenerator<'a> {
 
     fn module_function(
         &mut self,
-        name: &'a str,
+        name: &'a EcoString,
         args: &'a [TypedArg],
         return_type: &'a Arc<Type>,
     ) -> Output<'a> {
@@ -565,7 +567,7 @@ impl<'a> TypeScriptGenerator<'a> {
 
         Ok(docvec![
             "export function ",
-            super::maybe_escape_identifier_doc(name),
+            super::maybe_escape_identifier(name),
             if generic_names.is_empty() {
                 super::nil()
             } else {
@@ -584,7 +586,7 @@ impl<'a> TypeScriptGenerator<'a> {
                             ]
                         }
                         Some(name) => docvec![
-                            super::maybe_escape_identifier_doc(name),
+                            super::maybe_escape_identifier(name),
                             ": ",
                             self.print_type_with_generic_usages(&a.type_, &generic_usages)
                         ],
