@@ -8,8 +8,10 @@ use lsp_types::{
     TextEdit,
 };
 use strum::IntoEnumIterator;
+use vec1::Vec1;
 
 use crate::{
+    Result,
     ast::{
         self, Arg, CallArg, Definition, Function, FunctionLiteralKind, Pattern, Publicity,
         TypedExpr,
@@ -18,21 +20,20 @@ use crate::{
     io::{BeamCompiler, CommandExecutor, FileSystemReader, FileSystemWriter},
     line_numbers::LineNumbers,
     type_::{
-        self, collapse_links, error::VariableOrigin, pretty::Printer, FieldMap, ModuleInterface,
-        PreludeType, RecordAccessor, Type, TypeConstructor, ValueConstructorVariant,
-        PRELUDE_MODULE_NAME,
+        self, FieldMap, ModuleInterface, PRELUDE_MODULE_NAME, PreludeType, RecordAccessor, Type,
+        TypeConstructor, ValueConstructorVariant, collapse_links, error::VariableOrigin,
+        pretty::Printer,
     },
-    Result,
 };
 
 use super::{
+    DownloadDependencies, MakeLocker,
     compiler::LspProjectCompiler,
     edits::{
-        add_newlines_after_import, get_import, get_import_edit,
-        position_of_first_definition_if_import, Newlines,
+        Newlines, add_newlines_after_import, get_import, get_import_edit,
+        position_of_first_definition_if_import,
     },
     files::FileSystemProxy,
-    DownloadDependencies, MakeLocker,
 };
 
 // Represents the kind/specificity of completion that is being requested.
@@ -1070,7 +1071,7 @@ impl<'ast> ast::visit::Visit<'ast> for LocalCompletion<'_> {
         _: &'ast Arc<Type>,
         _: &'ast FunctionLiteralKind,
         args: &'ast [ast::TypedArg],
-        body: &'ast [ast::TypedStatement],
+        body: &'ast Vec1<ast::TypedStatement>,
         _: &'ast Option<ast::TypeAst>,
     ) {
         self.visit_fn_args(args);
