@@ -1,10 +1,11 @@
 use crate::{
+    Error, Result, Warning,
     analyse::name::correct_name_case,
     ast::{
-        CustomType, Definition, ModuleConstant, SrcSpan, TypedArg, TypedExpr, TypedFunction,
-        TypedModule, TypedPattern,
+        self, ArgNames, CustomType, Definition, DefinitionLocation, ModuleConstant, Pattern,
+        SrcSpan, TypedArg, TypedExpr, TypedFunction, TypedModule, TypedPattern,
     },
-    build::{type_constructor_from_modules, Located, Module, UnqualifiedImport},
+    build::{Located, Module, UnqualifiedImport, type_constructor_from_modules},
     config::PackageConfig,
     io::{BeamCompiler, CommandExecutor, FileSystemReader, FileSystemWriter},
     language_server::{
@@ -13,31 +14,36 @@ use crate::{
     line_numbers::LineNumbers,
     paths::ProjectPaths,
     type_::{
-        self, printer::Printer, Deprecation, ModuleInterface, Type, TypeConstructor,
-        ValueConstructorVariant,
+        self, Deprecation, ModuleInterface, Type, TypeConstructor, ValueConstructor,
+        ValueConstructorVariant, error::VariableOrigin, printer::Printer,
     },
-    Error, Result, Warning,
 };
 use camino::Utf8PathBuf;
 use ecow::EcoString;
 use itertools::Itertools;
 use lsp::CodeAction;
 use lsp_types::{
-    self as lsp, DocumentSymbol, Hover, HoverContents, MarkedString, Position, Range,
-    SignatureHelp, SymbolKind, SymbolTag, TextEdit, Url,
+    self as lsp, DocumentSymbol, Hover, HoverContents, MarkedString, Position,
+    PrepareRenameResponse, Range, SignatureHelp, SymbolKind, SymbolTag, TextEdit, Url,
+    WorkspaceEdit,
 };
-use std::sync::Arc;
+use std::{collections::HashSet, sync::Arc};
 
 use super::{
+    DownloadDependencies, MakeLocker,
     code_action::{
-        code_action_add_missing_patterns, code_action_convert_qualified_constructor_to_unqualified,
+        AddAnnotations, CodeActionBuilder, ConvertFromUse, ConvertToFunctionCall, ConvertToPipe,
+        ConvertToUse, ExpandFunctionCapture, ExtractVariable, FillInMissingLabelledArgs,
+        GenerateDynamicDecoder, GenerateFunction, GenerateJsonEncoder, InlineVariable,
+        InterpolateString, LetAssertToCase, PatternMatchOnValue, RedundantTupleInCaseSubject,
+        UseLabelShorthandSyntax, code_action_add_missing_patterns,
+        code_action_convert_qualified_constructor_to_unqualified,
         code_action_convert_unqualified_constructor_to_qualified, code_action_import_module,
-        code_action_inexhaustive_let_to_case, AddAnnotations, CodeActionBuilder, DesugarUse,
-        ExpandFunctionCapture, ExtractVariable, FillInMissingLabelledArgs, GenerateDynamicDecoder,
-        LabelShorthandSyntax, LetAssertToCase, RedundantTupleInCaseSubject, TurnIntoUse,
+        code_action_inexhaustive_let_to_case,
     },
     completer::Completer,
-    signature_help, src_span_to_lsp_range, DownloadDependencies, MakeLocker,
+    rename::{VariableRenameKind, rename_local_variable},
+    signature_help, src_span_to_lsp_range,
 };
 
 #[derive(Debug, PartialEq, Eq)]
@@ -75,7 +81,7 @@ pub struct LanguageServerEngine<IO, Reporter> {
 
     /// Used to know if to show the "View on HexDocs" link
     /// when hovering on an imported value
-    hex_deps: std::collections::HashSet<EcoString>,
+    hex_deps: HashSet<EcoString>,
 }
 
 impl<'a, IO, Reporter> LanguageServerEngine<IO, Reporter>
@@ -178,31 +184,64 @@ where
                 None => return Ok(None),
             };
 
-            let location = match node
-                .definition_location(this.compiler.project_compiler.get_importable_modules())
-            {
-                Some(location) => location,
-                None => return Ok(None),
+            let Some(location) =
+                node.definition_location(this.compiler.project_compiler.get_importable_modules())
+            else {
+                return Ok(None);
             };
 
-            let (uri, line_numbers) = match location.module {
-                None => (params.text_document.uri, &line_numbers),
-                Some(name) => {
-                    let module = match this.compiler.get_source(name) {
-                        Some(module) => module,
-                        _ => return Ok(None),
-                    };
-                    let url = Url::parse(&format!("file:///{}", &module.path))
-                        .expect("goto definition URL parse");
-                    (url, &module.line_numbers)
-                }
-            };
-            let range = src_span_to_lsp_range(location.span, line_numbers);
-
-            Ok(Some(lsp::Location { uri, range }))
+            Ok(this.definition_location_to_lsp_location(&line_numbers, &params, location))
         })
     }
 
+    pub(crate) fn goto_type_definition(
+        &mut self,
+        params: lsp_types::GotoDefinitionParams,
+    ) -> Response<Vec<lsp::Location>> {
+        self.respond(|this| {
+            let params = params.text_document_position_params;
+            let (line_numbers, node) = match this.node_at_position(&params) {
+                Some(location) => location,
+                None => return Ok(vec![]),
+            };
+
+            let Some(locations) = node
+                .type_definition_locations(this.compiler.project_compiler.get_importable_modules())
+            else {
+                return Ok(vec![]);
+            };
+
+            let locations = locations
+                .into_iter()
+                .filter_map(|location| {
+                    this.definition_location_to_lsp_location(&line_numbers, &params, location)
+                })
+                .collect_vec();
+
+            Ok(locations)
+        })
+    }
+
+    fn definition_location_to_lsp_location(
+        &self,
+        line_numbers: &LineNumbers,
+        params: &lsp_types::TextDocumentPositionParams,
+        location: DefinitionLocation,
+    ) -> Option<lsp::Location> {
+        let (uri, line_numbers) = match location.module {
+            None => (params.text_document.uri.clone(), line_numbers),
+            Some(name) => {
+                let module = self.compiler.get_source(&name)?;
+                let url = Url::parse(&format!("file:///{}", &module.path))
+                    .expect("goto definition URL parse");
+                (url, &module.line_numbers)
+            }
+        };
+        let range = src_span_to_lsp_range(location.span, line_numbers);
+
+        Some(lsp::Location { uri, range })
+    }
+
     pub fn completion(
         &mut self,
         params: lsp::TextDocumentPositionParams,
@@ -283,6 +322,15 @@ where
                 Located::Annotation(_, _) => Some(completer.completion_types()),
 
                 Located::Label(_, _) => None,
+
+                Located::ModuleName {
+                    layer: ast::Layer::Type,
+                    ..
+                } => Some(completer.completion_types()),
+                Located::ModuleName {
+                    layer: ast::Layer::Value,
+                    ..
+                } => Some(completer.completion_values()),
             };
 
             Ok(completions)
@@ -328,13 +376,22 @@ where
             actions.extend(LetAssertToCase::new(module, &lines, &params).code_actions());
             actions
                 .extend(RedundantTupleInCaseSubject::new(module, &lines, &params).code_actions());
-            actions.extend(LabelShorthandSyntax::new(module, &lines, &params).code_actions());
+            actions.extend(UseLabelShorthandSyntax::new(module, &lines, &params).code_actions());
             actions.extend(FillInMissingLabelledArgs::new(module, &lines, &params).code_actions());
-            actions.extend(DesugarUse::new(module, &lines, &params).code_actions());
-            actions.extend(TurnIntoUse::new(module, &lines, &params).code_actions());
+            actions.extend(ConvertFromUse::new(module, &lines, &params).code_actions());
+            actions.extend(ConvertToUse::new(module, &lines, &params).code_actions());
             actions.extend(ExpandFunctionCapture::new(module, &lines, &params).code_actions());
+            actions.extend(InterpolateString::new(module, &lines, &params).code_actions());
             actions.extend(ExtractVariable::new(module, &lines, &params).code_actions());
+            actions.extend(GenerateFunction::new(module, &lines, &params).code_actions());
+            actions.extend(ConvertToPipe::new(module, &lines, &params).code_actions());
+            actions.extend(ConvertToFunctionCall::new(module, &lines, &params).code_actions());
+            actions.extend(
+                PatternMatchOnValue::new(module, &lines, &params, &this.compiler).code_actions(),
+            );
+            actions.extend(InlineVariable::new(module, &lines, &params).code_actions());
             GenerateDynamicDecoder::new(module, &lines, &params, &mut actions).code_actions();
+            GenerateJsonEncoder::new(module, &lines, &params, &mut actions).code_actions();
             AddAnnotations::new(module, &lines, &params).code_action(&mut actions);
             Ok(if actions.is_empty() {
                 None
@@ -489,6 +546,159 @@ where
         })
     }
 
+    pub fn prepare_rename(
+        &mut self,
+        params: lsp::TextDocumentPositionParams,
+    ) -> Response<Option<PrepareRenameResponse>> {
+        self.respond(|this| {
+            let (lines, found) = match this.node_at_position(&params) {
+                Some(value) => value,
+                None => return Ok(None),
+            };
+
+            let success_response = |location| {
+                Some(PrepareRenameResponse::Range(src_span_to_lsp_range(
+                    location, &lines,
+                )))
+            };
+
+            Ok(match found {
+                Located::Expression(TypedExpr::Var {
+                    constructor:
+                        ValueConstructor {
+                            variant: ValueConstructorVariant::LocalVariable { origin, location },
+                            ..
+                        },
+                    ..
+                })
+                | Located::Pattern(Pattern::Variable {
+                    origin, location, ..
+                }) => match origin {
+                    VariableOrigin::Variable(_)
+                    | VariableOrigin::AssignmentPattern
+                    | VariableOrigin::LabelShorthand(_) => success_response(*location),
+                    VariableOrigin::Generated => None,
+                },
+                Located::Pattern(Pattern::VarUsage { constructor, .. }) => constructor
+                    .as_ref()
+                    .and_then(|constructor| match &constructor.variant {
+                        ValueConstructorVariant::LocalVariable { origin, location } => match origin
+                        {
+                            VariableOrigin::Variable(_)
+                            | VariableOrigin::AssignmentPattern
+                            | VariableOrigin::LabelShorthand(_) => success_response(*location),
+                            VariableOrigin::Generated => None,
+                        },
+                        _ => None,
+                    }),
+                Located::Pattern(Pattern::Assign { location, .. }) => success_response(*location),
+                Located::Arg(arg) => match &arg.names {
+                    ArgNames::Named { location, .. }
+                    | ArgNames::NamedLabelled {
+                        name_location: location,
+                        ..
+                    } => success_response(*location),
+                    ArgNames::Discard { .. } | ArgNames::LabelledDiscard { .. } => None,
+                },
+                _ => None,
+            })
+        })
+    }
+
+    pub fn rename(&mut self, params: lsp::RenameParams) -> Response<Option<WorkspaceEdit>> {
+        self.respond(|this| {
+            let position = &params.text_document_position;
+
+            let (lines, found) = match this.node_at_position(position) {
+                Some(value) => value,
+                None => return Ok(None),
+            };
+
+            let Some(module) = this.module_for_uri(&position.text_document.uri) else {
+                return Ok(None);
+            };
+
+            Ok(match found {
+                Located::Expression(TypedExpr::Var {
+                    constructor:
+                        ValueConstructor {
+                            variant: ValueConstructorVariant::LocalVariable { location, origin },
+                            ..
+                        },
+                    ..
+                })
+                | Located::Pattern(Pattern::Variable {
+                    location, origin, ..
+                }) => match origin {
+                    VariableOrigin::Variable(_) | VariableOrigin::AssignmentPattern => {
+                        rename_local_variable(
+                            module,
+                            &lines,
+                            &params,
+                            *location,
+                            VariableRenameKind::Variable,
+                        )
+                    }
+                    VariableOrigin::LabelShorthand(_) => rename_local_variable(
+                        module,
+                        &lines,
+                        &params,
+                        *location,
+                        VariableRenameKind::LabelShorthand,
+                    ),
+                    VariableOrigin::Generated => None,
+                },
+                Located::Pattern(Pattern::VarUsage { constructor, .. }) => constructor
+                    .as_ref()
+                    .and_then(|constructor| match &constructor.variant {
+                        ValueConstructorVariant::LocalVariable { location, origin } => match origin
+                        {
+                            VariableOrigin::Variable(_) | VariableOrigin::AssignmentPattern => {
+                                rename_local_variable(
+                                    module,
+                                    &lines,
+                                    &params,
+                                    *location,
+                                    VariableRenameKind::Variable,
+                                )
+                            }
+                            VariableOrigin::LabelShorthand(_) => rename_local_variable(
+                                module,
+                                &lines,
+                                &params,
+                                *location,
+                                VariableRenameKind::LabelShorthand,
+                            ),
+                            VariableOrigin::Generated => None,
+                        },
+                        _ => None,
+                    }),
+                Located::Pattern(Pattern::Assign { location, .. }) => rename_local_variable(
+                    module,
+                    &lines,
+                    &params,
+                    *location,
+                    VariableRenameKind::Variable,
+                ),
+                Located::Arg(arg) => match &arg.names {
+                    ArgNames::Named { location, .. }
+                    | ArgNames::NamedLabelled {
+                        name_location: location,
+                        ..
+                    } => rename_local_variable(
+                        module,
+                        &lines,
+                        &params,
+                        *location,
+                        VariableRenameKind::Variable,
+                    ),
+                    ArgNames::Discard { .. } | ArgNames::LabelledDiscard { .. } => None,
+                },
+                _ => None,
+            })
+        })
+    }
+
     fn respond<T>(&mut self, handler: impl FnOnce(&mut Self) -> Result<T>) -> Response<T> {
         let result = handler(self);
         let warnings = self.take_warnings();
@@ -528,6 +738,17 @@ where
                 Located::ModuleStatement(Definition::ModuleConstant(constant)) => {
                     Some(hover_for_module_constant(constant, lines, module))
                 }
+                Located::ModuleStatement(Definition::Import(import)) => {
+                    let Some(module) = this.compiler.get_module_interface(&import.module) else {
+                        return Ok(None);
+                    };
+                    Some(hover_for_module(
+                        module,
+                        import.location,
+                        &lines,
+                        &this.hex_deps,
+                    ))
+                }
                 Located::ModuleStatement(_) => None,
                 Located::UnqualifiedImport(UnqualifiedImport {
                     name,
@@ -627,6 +848,12 @@ Unused labelled fields:
                 Located::Label(location, type_) => {
                     Some(hover_for_label(location, type_, lines, module))
                 }
+                Located::ModuleName { location, name, .. } => {
+                    let Some(module) = this.compiler.get_module_interface(name) else {
+                        return Ok(None);
+                    };
+                    Some(hover_for_module(module, location, &lines, &this.hex_deps))
+                }
             })
         })
     }
@@ -933,7 +1160,7 @@ fn hover_for_expression(
     expression: &TypedExpr,
     line_numbers: LineNumbers,
     module: &Module,
-    hex_deps: &std::collections::HashSet<EcoString>,
+    hex_deps: &HashSet<EcoString>,
 ) -> Hover {
     let documentation = expression.get_documentation().unwrap_or_default();
 
@@ -958,7 +1185,7 @@ fn hover_for_expression(
 }
 
 fn hover_for_imported_value(
-    value: &type_::ValueConstructor,
+    value: &ValueConstructor,
     location: &SrcSpan,
     line_numbers: LineNumbers,
     hex_module_imported_from: Option<&ModuleInterface>,
@@ -968,7 +1195,7 @@ fn hover_for_imported_value(
     let documentation = value.get_documentation().unwrap_or_default();
 
     let link_section = hex_module_imported_from.map_or("".to_string(), |m| {
-        format_hexdocs_link_section(m.package.as_str(), m.name.as_str(), name)
+        format_hexdocs_link_section(m.package.as_str(), m.name.as_str(), Some(name))
     });
 
     // Show the type of the hovered node to the user
@@ -985,6 +1212,34 @@ fn hover_for_imported_value(
     }
 }
 
+fn hover_for_module(
+    module: &ModuleInterface,
+    location: SrcSpan,
+    line_numbers: &LineNumbers,
+    hex_deps: &HashSet<EcoString>,
+) -> Hover {
+    let documentation = module.documentation.join("\n");
+    let name = &module.name;
+
+    let link_section = if hex_deps.contains(&module.package) {
+        format_hexdocs_link_section(&module.package, name, None)
+    } else {
+        String::new()
+    };
+
+    let contents = format!(
+        "```gleam
+{name}
+```
+{documentation}
+{link_section}",
+    );
+    Hover {
+        contents: HoverContents::Scalar(MarkedString::String(contents)),
+        range: Some(src_span_to_lsp_range(location, line_numbers)),
+    }
+}
+
 // Returns true if any part of either range overlaps with the other.
 pub fn overlaps(a: Range, b: Range) -> bool {
     position_within(a.start, b)
@@ -1003,6 +1258,8 @@ fn position_within(position: Position, range: Range) -> bool {
     position >= range.start && position <= range.end
 }
 
+/// Builds the code action to assign an unused value to `_`.
+///
 fn code_action_unused_values(
     module: &Module,
     line_numbers: &LineNumbers,
@@ -1062,6 +1319,8 @@ fn code_action_unused_values(
     }
 }
 
+/// Code action to remove unused imports.
+///
 fn code_action_unused_imports(
     module: &Module,
     line_numbers: &LineNumbers,
@@ -1212,8 +1471,15 @@ fn get_expr_qualified_name(expression: &TypedExpr) -> Option<(&EcoString, &EcoSt
     }
 }
 
-fn format_hexdocs_link_section(package_name: &str, module_name: &str, name: &str) -> String {
-    let link = format!("https://hexdocs.pm/{package_name}/{module_name}.html#{name}");
+fn format_hexdocs_link_section(
+    package_name: &str,
+    module_name: &str,
+    name: Option<&str>,
+) -> String {
+    let link = match name {
+        Some(name) => format!("https://hexdocs.pm/{package_name}/{module_name}.html#{name}"),
+        None => format!("https://hexdocs.pm/{package_name}/{module_name}.html"),
+    };
     format!("\nView on [HexDocs]({link})")
 }
 
@@ -1221,7 +1487,7 @@ fn get_hexdocs_link_section(
     module_name: &str,
     name: &str,
     ast: &TypedModule,
-    hex_deps: &std::collections::HashSet<EcoString>,
+    hex_deps: &HashSet<EcoString>,
 ) -> Option<String> {
     let package_name = ast.definitions.iter().find_map(|def| match def {
         Definition::Import(p) if p.module == module_name && hex_deps.contains(&p.package) => {
@@ -1230,7 +1496,11 @@ fn get_hexdocs_link_section(
         _ => None,
     })?;
 
-    Some(format_hexdocs_link_section(package_name, module_name, name))
+    Some(format_hexdocs_link_section(
+        package_name,
+        module_name,
+        Some(name),
+    ))
 }
 
 /// Converts the source start position of a documentation comment's contents into
