@@ -7,8 +7,8 @@ use lsp_types::{
     self as lsp,
     notification::{DidChangeTextDocument, DidCloseTextDocument, DidSaveTextDocument},
     request::{
-        CodeActionRequest, Completion, DocumentSymbolRequest, Formatting, HoverRequest,
-        SignatureHelpRequest,
+        CodeActionRequest, Completion, DocumentSymbolRequest, Formatting, GotoTypeDefinition,
+        HoverRequest, PrepareRenameRequest, Rename, SignatureHelpRequest,
     },
 };
 use std::time::Duration;
@@ -24,10 +24,13 @@ pub enum Request {
     Format(lsp::DocumentFormattingParams),
     Hover(lsp::HoverParams),
     GoToDefinition(lsp::GotoDefinitionParams),
+    GoToTypeDefinition(lsp::GotoDefinitionParams),
     Completion(lsp::CompletionParams),
     CodeAction(lsp::CodeActionParams),
     SignatureHelp(lsp::SignatureHelpParams),
     DocumentSymbol(lsp::DocumentSymbolParams),
+    PrepareRename(lsp::TextDocumentPositionParams),
+    Rename(lsp::RenameParams),
 }
 
 impl Request {
@@ -62,6 +65,18 @@ impl Request {
                 let params = cast_request::<DocumentSymbolRequest>(request);
                 Some(Message::Request(id, Request::DocumentSymbol(params)))
             }
+            "textDocument/rename" => {
+                let params = cast_request::<Rename>(request);
+                Some(Message::Request(id, Request::Rename(params)))
+            }
+            "textDocument/prepareRename" => {
+                let params = cast_request::<PrepareRenameRequest>(request);
+                Some(Message::Request(id, Request::PrepareRename(params)))
+            }
+            "textDocument/typeDefinition" => {
+                let params = cast_request::<GotoTypeDefinition>(request);
+                Some(Message::Request(id, Request::GoToTypeDefinition(params)))
+            }
             _ => None,
         }
     }
