@@ -11,9 +11,9 @@ use crate::{
     build::Origin,
     line_numbers::LineNumbers,
     type_::{
-        self, expression::Implementations, Deprecation, ModuleInterface, Type, TypeConstructor,
+        self, Deprecation, ModuleInterface, Opaque, Type, TypeAliasConstructor, TypeConstructor,
         TypeValueConstructor, TypeValueConstructorField, TypeVariantConstructors, ValueConstructor,
-        ValueConstructorVariant,
+        ValueConstructorVariant, expression::Implementations, prelude,
     },
     uid::UniqueIdGenerator,
 };
@@ -64,6 +64,9 @@ fn constant_module(constant: TypedConstant) -> ModuleInterface {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     }
 }
 
@@ -98,6 +101,9 @@ fn empty_module() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -121,6 +127,9 @@ fn with_line_numbers() {
         ),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -152,6 +161,9 @@ fn module_with_private_type() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -183,6 +195,9 @@ fn module_with_app_type() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -214,6 +229,9 @@ fn module_with_fn_type() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -245,6 +263,9 @@ fn module_with_tuple_type() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -282,6 +303,9 @@ fn module_with_generic_type() {
             line_numbers: LineNumbers::new(""),
             src_path: "some_path".into(),
             minimum_required_version: Version::new(0, 1, 0),
+            type_aliases: HashMap::new(),
+            documentation: Vec::new(),
+            contains_echo: false,
         }
     }
 
@@ -319,6 +343,9 @@ fn module_with_type_links() {
             line_numbers: LineNumbers::new(""),
             src_path: "some_path".into(),
             minimum_required_version: Version::new(0, 1, 0),
+            type_aliases: HashMap::new(),
+            documentation: Vec::new(),
+            contains_echo: false,
         }
     }
 
@@ -356,6 +383,9 @@ fn module_with_type_constructor_documentation() {
             line_numbers: LineNumbers::new(""),
             src_path: "some_path".into(),
             minimum_required_version: Version::new(0, 1, 0),
+            type_aliases: HashMap::new(),
+            documentation: Vec::new(),
+            contains_echo: false,
         }
     }
 
@@ -396,6 +426,9 @@ fn module_with_type_constructor_origin() {
             line_numbers: LineNumbers::new(""),
             src_path: "some_path".into(),
             minimum_required_version: Version::new(0, 1, 0),
+            type_aliases: HashMap::new(),
+            documentation: Vec::new(),
+            contains_echo: false,
         }
     }
 
@@ -418,7 +451,9 @@ fn module_type_to_constructors_mapping() {
                 variants: vec![TypeValueConstructor {
                     name: "One".into(),
                     parameters: vec![],
+                    documentation: Some("Some documentation".into()),
                 }],
+                opaque: Opaque::NotOpaque,
             },
         )]
         .into(),
@@ -427,6 +462,9 @@ fn module_type_to_constructors_mapping() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -475,6 +513,9 @@ fn module_fn_value() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -524,6 +565,9 @@ fn deprecated_module_fn_value() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -571,6 +615,9 @@ fn private_module_fn_value() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -620,6 +667,9 @@ fn module_fn_value_regression() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -668,6 +718,9 @@ fn module_fn_value_with_field_map() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -697,13 +750,13 @@ fn record_value() {
                     name: "one".into(),
                     module: "themodule".into(),
                     field_map: None,
-                    arity: random.gen(),
-                    variants_count: random.gen(),
+                    arity: random.r#gen(),
+                    variants_count: random.r#gen(),
                     location: SrcSpan {
-                        start: random.gen(),
-                        end: random.gen(),
+                        start: random.r#gen(),
+                        end: random.r#gen(),
                     },
-                    variant_index: random.gen(),
+                    variant_index: random.r#gen(),
                 },
             },
         )]
@@ -711,6 +764,9 @@ fn record_value() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -740,15 +796,16 @@ fn record_value_with_field_map() {
                     module: "themodule".into(),
                     name: "one".into(),
                     field_map: Some(FieldMap {
-                        arity: random.gen(),
-                        fields: [("ok".into(), random.gen()), ("ko".into(), random.gen())].into(),
+                        arity: random.r#gen(),
+                        fields: [("ok".into(), random.r#gen()), ("ko".into(), random.r#gen())]
+                            .into(),
                     }),
-                    arity: random.gen(),
-                    variants_count: random.gen(),
-                    variant_index: random.gen(),
+                    arity: random.r#gen(),
+                    variants_count: random.r#gen(),
+                    variant_index: random.r#gen(),
                     location: SrcSpan {
-                        start: random.gen(),
-                        end: random.gen(),
+                        start: random.r#gen(),
+                        end: random.r#gen(),
                     },
                 },
             },
@@ -757,6 +814,9 @@ fn record_value_with_field_map() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -825,6 +885,80 @@ fn accessors() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
+    };
+
+    assert_eq!(roundtrip(&module), module);
+}
+
+#[test]
+fn private_accessors() {
+    let accessors1 = [
+        (
+            "a".into(),
+            RecordAccessor {
+                index: 6,
+                label: "siiixxx".into(),
+                type_: type_::nil(),
+            },
+        ),
+        (
+            "a".into(),
+            RecordAccessor {
+                index: 5,
+                label: "fiveee".into(),
+                type_: type_::float(),
+            },
+        ),
+    ];
+
+    let accessors2 = [(
+        "a".into(),
+        RecordAccessor {
+            index: 1,
+            label: "ok".into(),
+            type_: type_::float(),
+        },
+    )];
+
+    let module = ModuleInterface {
+        warnings: vec![],
+        is_internal: false,
+        package: "some_package".into(),
+        origin: Origin::Src,
+        name: "a".into(),
+        types: HashMap::new(),
+        types_value_constructors: HashMap::new(),
+        contains_echo: false,
+        values: HashMap::new(),
+        accessors: [
+            (
+                "one".into(),
+                AccessorsMap {
+                    publicity: Publicity::Private,
+                    type_: type_::int(),
+                    shared_accessors: accessors1.clone().into(),
+                    variant_specific_accessors: vec![accessors1.into()],
+                },
+            ),
+            (
+                "two".into(),
+                AccessorsMap {
+                    publicity: Publicity::Public,
+                    type_: type_::int(),
+                    shared_accessors: accessors2.clone().into(),
+                    variant_specific_accessors: vec![accessors2.into()],
+                },
+            ),
+        ]
+        .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1047,6 +1181,9 @@ fn constant_var() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1239,6 +1376,9 @@ fn deprecated_type() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -1286,6 +1426,32 @@ fn module_fn_value_with_external_implementations() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
+    };
+
+    assert_eq!(roundtrip(&module), module);
+}
+
+#[test]
+fn module_containing_echo() {
+    let module = ModuleInterface {
+        warnings: vec![],
+        is_internal: false,
+        package: "some_package".into(),
+        origin: Origin::Src,
+        name: "a/b/c".into(),
+        types: HashMap::new(),
+        types_value_constructors: HashMap::new(),
+        accessors: HashMap::new(),
+        values: [].into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: true,
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1336,6 +1502,9 @@ fn internal_module_fn() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1389,6 +1558,9 @@ fn internal_annotated_module_fn() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1413,15 +1585,20 @@ fn type_variable_ids_in_constructors_are_shared() {
                     parameters: vec![
                         TypeValueConstructorField {
                             type_: type_::generic_var(6),
+                            label: None,
                         },
                         TypeValueConstructorField {
                             type_: type_::int(),
+                            label: None,
                         },
                         TypeValueConstructorField {
                             type_: type_::tuple(vec![type_::generic_var(4), type_::generic_var(5)]),
+                            label: None,
                         },
                     ],
+                    documentation: None,
                 }],
+                opaque: Opaque::NotOpaque,
             },
         )]),
         accessors: HashMap::new(),
@@ -1429,6 +1606,9 @@ fn type_variable_ids_in_constructors_are_shared() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
     };
 
     let expected = HashMap::from([(
@@ -1440,15 +1620,20 @@ fn type_variable_ids_in_constructors_are_shared() {
                 parameters: vec![
                     TypeValueConstructorField {
                         type_: type_::generic_var(0),
+                        label: None,
                     },
                     TypeValueConstructorField {
                         type_: type_::int(),
+                        label: None,
                     },
                     TypeValueConstructorField {
                         type_: type_::tuple(vec![type_::generic_var(1), type_::generic_var(2)]),
+                        label: None,
                     },
                 ],
+                documentation: None,
             }],
+            opaque: Opaque::NotOpaque,
         },
     )]);
 
@@ -1461,6 +1646,7 @@ fn type_with_inferred_variant() {
         warnings: vec![],
         is_internal: false,
         package: "some_package".into(),
+        contains_echo: false,
         origin: Origin::Src,
         name: "a/b".into(),
         types: [(
@@ -1489,6 +1675,103 @@ fn type_with_inferred_variant() {
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
         minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
     };
     assert_eq!(roundtrip(&module), module);
 }
+
+#[test]
+fn module_with_type_aliases() {
+    let module = ModuleInterface {
+        warnings: vec![],
+        is_internal: false,
+        package: "some_package".into(),
+        origin: Origin::Src,
+        name: "some_module".into(),
+        types: HashMap::new(),
+        types_value_constructors: HashMap::new(),
+        values: HashMap::new(),
+        accessors: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: [(
+            "MyAlias".into(),
+            TypeAliasConstructor {
+                publicity: Publicity::Public,
+                module: "some_module".into(),
+                type_: prelude::int(),
+                arity: 1,
+                deprecation: Deprecation::NotDeprecated,
+                documentation: Some("Some documentation".into()),
+                origin: Default::default(),
+            },
+        )]
+        .into(),
+        documentation: Vec::new(),
+        contains_echo: false,
+    };
+    assert_eq!(roundtrip(&module), module);
+}
+
+#[test]
+fn module_with_documentation() {
+    let module = ModuleInterface {
+        warnings: vec![],
+        is_internal: false,
+        package: "some_package".into(),
+        origin: Origin::Src,
+        name: "some_module".into(),
+        types: HashMap::new(),
+        types_value_constructors: HashMap::new(),
+        values: HashMap::new(),
+        accessors: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: vec![
+            "This is a line of documentation".into(),
+            "And here is another".into(),
+            "And finally, a third".into(),
+        ],
+        contains_echo: false,
+    };
+    assert_eq!(roundtrip(&module), module);
+}
+
+#[test]
+fn module_with_opaque_type() {
+    let module = ModuleInterface {
+        warnings: vec![],
+        is_internal: false,
+        package: "some_package".into(),
+        origin: Origin::Src,
+        name: "a".into(),
+        types: HashMap::new(),
+        types_value_constructors: [(
+            "SomeType".into(),
+            TypeVariantConstructors {
+                type_parameters_ids: vec![0, 1, 2],
+                variants: vec![TypeValueConstructor {
+                    name: "One".into(),
+                    parameters: vec![],
+                    documentation: Some("Some documentation".into()),
+                }],
+                opaque: Opaque::Opaque,
+            },
+        )]
+        .into(),
+        accessors: HashMap::new(),
+        values: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
+        type_aliases: HashMap::new(),
+        documentation: Vec::new(),
+        contains_echo: false,
+    };
+
+    assert_eq!(roundtrip(&module), module);
+}
