@@ -135,7 +135,11 @@ pub fn compile_package(
     };
     module.attach_doc_and_module_comments();
     let package: Package = package_from_module(module);
-    serde_json::to_string_pretty(&PackageInterface::from_package(&package)).expect("to json")
+    serde_json::to_string_pretty(&PackageInterface::from_package(
+        &package,
+        &Default::default(),
+    ))
+    .expect("to json")
 }
 
 fn package_from_module(module: Module) -> Package {
@@ -167,10 +171,13 @@ fn package_from_module(module: Module) -> Package {
             erlang: ErlangConfig::default(),
             javascript: JavaScriptConfig::default(),
             target: Target::Erlang,
-            internal_modules: Some(vec![GlobBuilder::new("internals/*")
-                .build()
-                .expect("internals glob")]),
+            internal_modules: Some(vec![
+                GlobBuilder::new("internals/*")
+                    .build()
+                    .expect("internals glob"),
+            ]),
         },
+        cached_module_names: Vec::new(),
         modules: vec![module],
     }
 }
@@ -305,3 +312,28 @@ pub type Box(a, b) {
 pub fn internal_modules_are_not_exported() {
     assert_package_interface_with_name!("internals/internal_module", "pub fn main() { 1 }");
 }
+
+#[test]
+pub fn labelled_function_parameters() {
+    assert_package_interface!(
+        r#"
+pub fn fold(list: List(a), from acc: b, with f: fn(a, b) -> b) -> b {
+  todo
+}
+"#
+    );
+}
+
+#[test]
+pub fn constructors_with_documentation() {
+    assert_package_interface!(
+        r#"
+pub type Wibble {
+  /// This is the Wibble variant. It contains some example data.
+  Wibble(Int)
+  /// This is the Wobble variant. It is a recursive type.
+  Wobble(Wibble)
+}
+"#
+    );
+}
