@@ -2,7 +2,7 @@ use pubgrub::range::Range;
 
 use crate::{
     analyse::TargetSupport,
-    ast::{Publicity, PIPE_VARIABLE},
+    ast::{PIPE_VARIABLE, Publicity},
     build::Target,
     error::edit_distance,
     uid::UniqueIdGenerator,
@@ -47,6 +47,8 @@ pub struct Environment<'a> {
     /// Mapping from types to constructor names in the current module (or the prelude)
     pub module_types_constructors: HashMap<EcoString, TypeVariantConstructors>,
 
+    pub module_type_aliases: HashMap<EcoString, TypeAliasConstructor>,
+
     /// Values defined in the current module (or the prelude)
     pub module_values: HashMap<EcoString, ValueConstructor>,
 
@@ -64,6 +66,9 @@ pub struct Environment<'a> {
     pub target_support: TargetSupport,
 
     pub names: Names,
+
+    /// Wether we ran into an `echo` or not while analysing the current module.
+    pub echo_found: bool,
 }
 
 impl<'a> Environment<'a> {
@@ -115,6 +120,8 @@ impl<'a> Environment<'a> {
             entity_usages: vec![HashMap::new()],
             target_support,
             names,
+            module_type_aliases: HashMap::new(),
+            echo_found: false,
         }
     }
 }
@@ -218,13 +225,19 @@ impl Environment<'_> {
 
     /// Insert a variable in the current scope.
     ///
-    pub fn insert_local_variable(&mut self, name: EcoString, location: SrcSpan, type_: Arc<Type>) {
+    pub fn insert_local_variable(
+        &mut self,
+        name: EcoString,
+        location: SrcSpan,
+        origin: VariableOrigin,
+        type_: Arc<Type>,
+    ) {
         let _ = self.scope.insert(
             name,
             ValueConstructor {
                 deprecation: Deprecation::NotDeprecated,
                 publicity: Publicity::Private,
-                variant: ValueConstructorVariant::LocalVariable { location },
+                variant: ValueConstructorVariant::LocalVariable { location, origin },
                 type_,
             },
         );
@@ -315,6 +328,28 @@ impl Environment<'_> {
         }
     }
 
+    /// Map a type alias in the current scope.
+    /// Errors if the module already has a type with that name, unless the type is from the
+    /// prelude.
+    ///
+    pub fn insert_type_alias(
+        &mut self,
+        type_name: EcoString,
+        info: TypeAliasConstructor,
+    ) -> Result<(), Error> {
+        let name = type_name.clone();
+        let location = info.origin;
+        match self.module_type_aliases.insert(type_name, info) {
+            None => Ok(()),
+            Some(prelude_type) if is_prelude_module(&prelude_type.module) => Ok(()),
+            Some(previous) => Err(Error::DuplicateTypeName {
+                name,
+                location,
+                previous_location: previous.origin,
+            }),
+        }
+    }
+
     pub fn assert_unique_type_name(
         &mut self,
         name: &EcoString,
@@ -534,13 +569,13 @@ impl Environment<'_> {
             Type::Var { type_ } => {
                 match type_.borrow().deref() {
                     TypeVar::Link { type_ } => {
-                        return self.instantiate(type_.clone(), ids, hydrator)
+                        return self.instantiate(type_.clone(), ids, hydrator);
                     }
 
                     TypeVar::Unbound { .. } => {
                         return Arc::new(Type::Var {
                             type_: type_.clone(),
-                        })
+                        });
                     }
 
                     TypeVar::Generic { id } => match ids.get(id) {
