@@ -1,6 +1,6 @@
 use super::{
-    expression::{ArgumentKind, CallKind},
     FieldAccessUsage,
+    expression::{ArgumentKind, CallKind},
 };
 use crate::{
     ast::{BinOp, Layer, SrcSpan, TodoKind},
@@ -443,6 +443,11 @@ pub enum Error {
         missing: Vec<EcoString>,
     },
 
+    /// A case expression is missing its body.
+    MissingCaseBody {
+        location: SrcSpan,
+    },
+
     /// Let assignment's pattern does not match all possible values of the type.
     InexhaustiveLetAssignment {
         location: SrcSpan,
@@ -563,7 +568,7 @@ pub enum Error {
         name: EcoString,
     },
 
-    /// Occers when all the variant types of a custom type are deprecated
+    /// Occurs when all the variant types of a custom type are deprecated
     ///
     /// ```gleam
     /// type Wibble {
@@ -582,6 +587,24 @@ pub enum Error {
     DeprecatedVariantOnDeprecatedType {
         location: SrcSpan,
     },
+
+    ErlangFloatUnsafe {
+        location: SrcSpan,
+    },
+
+    /// When the echo keyword is not followed by an expression to be printed.
+    /// The only place where echo is allowed to appear on its own is as a step
+    /// of a pipeline, otherwise omitting the expression will result in this
+    /// error. For example:
+    ///
+    /// ```gleam
+    /// call(echo, 1, 2)
+    /// //   ^^^^ Error!
+    /// ```
+    ///
+    EchoWithNoFollowingExpression {
+        location: SrcSpan,
+    },
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -916,6 +939,7 @@ pub enum FeatureKind {
     RecordAccessVariantInference,
     LetAssertWithMessage,
     VariantWithDeprecatedAnnotation,
+    JavaScriptUnalignedBitArray,
 }
 
 impl FeatureKind {
@@ -941,6 +965,8 @@ impl FeatureKind {
             FeatureKind::VariantWithDeprecatedAnnotation | FeatureKind::LetAssertWithMessage => {
                 Version::new(1, 7, 0)
             }
+
+            FeatureKind::JavaScriptUnalignedBitArray => Version::new(1, 9, 0),
         }
     }
 }
@@ -954,6 +980,9 @@ pub enum PanicPosition {
     /// argument must be a panic.
     LastFunctionArgument,
 
+    /// When the expression to be printed by echo panics.
+    EchoExpression,
+
     /// Any expression that doesn't fall in the previous two categories
     PreviousExpression,
 }
@@ -1027,6 +1056,7 @@ impl Error {
             | Error::InvalidExternalJavascriptModule { location, .. }
             | Error::InvalidExternalJavascriptFunction { location, .. }
             | Error::InexhaustiveCaseExpression { location, .. }
+            | Error::MissingCaseBody { location }
             | Error::InexhaustiveLetAssignment { location, .. }
             | Error::UnusedTypeAliasParameter { location, .. }
             | Error::DuplicateTypeParameter { location, .. }
@@ -1040,7 +1070,10 @@ impl Error {
             | Error::UseFnIncorrectArity { location, .. }
             | Error::BadName { location, .. }
             | Error::AllVariantsDeprecated { location }
-            | Error::DeprecatedVariantOnDeprecatedType { location } => location.start,
+            | Error::EchoWithNoFollowingExpression { location }
+            | Error::DeprecatedVariantOnDeprecatedType { location }
+            | Error::ErlangFloatUnsafe { location } => location.start,
+
             Error::UnknownLabels { unknown, .. } => {
                 unknown.iter().map(|(_, s)| s.start).min().unwrap_or(0)
             }
@@ -1329,12 +1362,16 @@ fn flip_unify_error_test() {
         UnifyError::CouldNotUnify {
             expected: crate::type_::int(),
             given: crate::type_::float(),
-            situation: Some(UnifyErrorSituation::CaseClauseMismatch),
+            situation: Some(UnifyErrorSituation::CaseClauseMismatch {
+                clause_location: SrcSpan::default()
+            }),
         },
         flip_unify_error(UnifyError::CouldNotUnify {
             expected: crate::type_::float(),
             given: crate::type_::int(),
-            situation: Some(UnifyErrorSituation::CaseClauseMismatch),
+            situation: Some(UnifyErrorSituation::CaseClauseMismatch {
+                clause_location: SrcSpan::default()
+            }),
         })
     );
 }
@@ -1364,7 +1401,9 @@ fn unify_enclosed_type_test() {
         Err(UnifyError::CouldNotUnify {
             expected: crate::type_::int(),
             given: crate::type_::float(),
-            situation: Some(UnifyErrorSituation::CaseClauseMismatch)
+            situation: Some(UnifyErrorSituation::CaseClauseMismatch {
+                clause_location: SrcSpan::default()
+            })
         }),
         unify_enclosed_type(
             crate::type_::int(),
@@ -1372,7 +1411,9 @@ fn unify_enclosed_type_test() {
             Err(UnifyError::CouldNotUnify {
                 expected: crate::type_::string(),
                 given: crate::type_::bits(),
-                situation: Some(UnifyErrorSituation::CaseClauseMismatch)
+                situation: Some(UnifyErrorSituation::CaseClauseMismatch {
+                    clause_location: SrcSpan::default()
+                })
             })
         )
     );
@@ -1437,7 +1478,9 @@ pub fn unify_wrong_returns(
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum UnifyErrorSituation {
     /// Clauses in a case expression were found to return different types.
-    CaseClauseMismatch,
+    CaseClauseMismatch {
+        clause_location: SrcSpan,
+    },
 
     /// A function was found to return a value that did not match its return
     /// annotation.
@@ -1480,7 +1523,7 @@ pub enum FunctionsMismatchReason {
 impl UnifyErrorSituation {
     pub fn description(&self) -> Option<&'static str> {
         match self {
-            Self::CaseClauseMismatch => Some(
+            Self::CaseClauseMismatch { clause_location: _ } => Some(
                 "This case clause was found to return a different type than the previous
 one, but all case clauses must return the same type.",
             ),
@@ -1545,8 +1588,8 @@ impl UnifyError {
         }
     }
 
-    pub fn case_clause_mismatch(self) -> Self {
-        self.with_unify_error_situation(UnifyErrorSituation::CaseClauseMismatch)
+    pub fn case_clause_mismatch(self, clause_location: SrcSpan) -> Self {
+        self.with_unify_error_situation(UnifyErrorSituation::CaseClauseMismatch { clause_location })
     }
 
     pub fn list_element_mismatch(self) -> Self {
@@ -1681,3 +1724,25 @@ pub fn check_javascript_int_safety(int_value: &BigInt, location: SrcSpan, proble
         problems.warning(Warning::JavaScriptIntUnsafe { location });
     }
 }
+
+/// When targeting Erlang, adds an error if the given Float value is outside the range
+/// -1.7976931348623157e308 to 1.7976931348623157e308 which is the allowed range for
+/// Erlang's floating point numbers
+///
+pub fn check_erlang_float_safety(
+    string_value: &EcoString,
+    location: SrcSpan,
+    problems: &mut Problems,
+) {
+    let erl_min_float = -1.7976931348623157e308f64;
+    let erl_max_float = 1.7976931348623157e308f64;
+
+    let float_value: f64 = string_value
+        .replace("_", "")
+        .parse()
+        .expect("Unable to parse string to floating point value");
+
+    if float_value < erl_min_float || float_value > erl_max_float {
+        problems.error(Error::ErlangFloatUnsafe { location });
+    }
+}
