@@ -2,8 +2,8 @@ use self::expression::CallKind;
 
 use super::*;
 use crate::ast::{
-    Assignment, AssignmentKind, ImplicitCallArgOrigin, Statement, TypedAssignment, UntypedExpr,
-    PIPE_VARIABLE,
+    FunctionLiteralKind, ImplicitCallArgOrigin, PIPE_VARIABLE, PipelineAssignmentKind, Statement,
+    TypedPipelineAssignment, UntypedExpr,
 };
 use vec1::Vec1;
 
@@ -13,15 +13,34 @@ pub(crate) struct PipeTyper<'a, 'b, 'c> {
     argument_type: Arc<Type>,
     argument_location: SrcSpan,
     location: SrcSpan,
-    assignments: Vec<TypedAssignment>,
+    first_value: TypedPipelineAssignment,
+    assignments: Vec<(TypedPipelineAssignment, PipelineAssignmentKind)>,
     expr_typer: &'a mut ExprTyper<'b, 'c>,
 }
 
 impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
+    fn new(expr_typer: &'a mut ExprTyper<'b, 'c>, size: usize, first: TypedExpr, end: u32) -> Self {
+        let first_type = first.type_();
+        let first_location = first.location();
+        let first_value = new_pipeline_assignment(expr_typer, first);
+        Self {
+            size,
+            expr_typer,
+            argument_type: first_type,
+            argument_location: first_location,
+            location: SrcSpan {
+                start: first_location.start,
+                end,
+            },
+            assignments: Vec::with_capacity(size),
+            first_value,
+        }
+    }
+
     pub fn infer(
         expr_typer: &'a mut ExprTyper<'b, 'c>,
         expressions: Vec1<UntypedExpr>,
-    ) -> Result<TypedExpr, Error> {
+    ) -> TypedExpr {
         // The scope is reset as pipelines are rewritten into a series of
         // assignments, and we don't want these variables to leak out of the
         // pipeline.
@@ -31,54 +50,45 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
         result
     }
 
-    fn run(
-        expr_typer: &'a mut ExprTyper<'b, 'c>,
-        expressions: Vec1<UntypedExpr>,
-    ) -> Result<TypedExpr, Error> {
+    fn run(expr_typer: &'a mut ExprTyper<'b, 'c>, expressions: Vec1<UntypedExpr>) -> TypedExpr {
         let size = expressions.len();
         let end = expressions.last().location().end;
         let mut expressions = expressions.into_iter();
-        let first = expr_typer.infer(expressions.next().expect("Empty pipeline in typer"))?;
-        let mut typer = Self {
-            size,
-            expr_typer,
-            argument_type: first.type_(),
-            argument_location: first.location(),
-            location: SrcSpan {
-                start: first.location().start,
-                end,
-            },
-            assignments: Vec::with_capacity(size),
+        let first = expressions.next().expect("Empty pipeline in typer");
+        let first_location = first.location();
+        let first = match expr_typer.infer(first) {
+            Ok(inferred) => inferred,
+            Err(e) => {
+                expr_typer.problems.error(e);
+                TypedExpr::Invalid {
+                    location: first_location,
+                    type_: expr_typer.new_unbound_var(),
+                }
+            }
         };
 
-        // No need to update self.argument_* as we set it above
-        typer.push_assignment_no_update(first);
-
-        // Perform the type checking
-        typer.infer_expressions(expressions)
+        Self::new(expr_typer, size, first, end).infer_expressions(expressions)
     }
 
     fn infer_expressions(
-        &mut self,
+        mut self,
         expressions: impl IntoIterator<Item = UntypedExpr>,
-    ) -> Result<TypedExpr, Error> {
-        let finally = self.infer_each_expression(expressions);
-
-        // Return any errors after clean-up
-        let finally = finally?;
+    ) -> TypedExpr {
+        let (finally, finally_kind) = self.infer_each_expression(expressions);
         let assignments = std::mem::take(&mut self.assignments);
-
-        Ok(TypedExpr::Pipeline {
-            assignments,
+        TypedExpr::Pipeline {
             location: self.location,
+            first_value: self.first_value,
+            assignments,
             finally: Box::new(finally),
-        })
+            finally_kind,
+        }
     }
 
     fn infer_each_expression(
         &mut self,
         expressions: impl IntoIterator<Item = UntypedExpr>,
-    ) -> Result<TypedExpr, Error> {
+    ) -> (TypedExpr, PipelineAssignmentKind) {
         let mut finally = None;
 
         for (i, call) in expressions.into_iter().enumerate() {
@@ -89,52 +99,109 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
 
             self.warn_if_call_first_argument_is_hole(&call);
 
-            let call = match call {
-                func @ UntypedExpr::Fn { location, .. } => {
+            let (kind, call) = match call {
+                func @ UntypedExpr::Fn { location, kind, .. } => {
                     let (func, args, return_type) = self.expr_typer.do_infer_call(
                         func.clone(),
                         vec![self.untyped_left_hand_value_variable_call_argument()],
                         location,
                         CallKind::Function,
                     );
-                    TypedExpr::Call {
-                        location,
-                        args,
-                        type_: return_type,
-                        fun: Box::new(func),
-                    }
+
+                    let kind = match kind {
+                        FunctionLiteralKind::Capture { hole } => {
+                            PipelineAssignmentKind::Hole { hole }
+                        }
+                        FunctionLiteralKind::Anonymous { .. } | FunctionLiteralKind::Use { .. } => {
+                            PipelineAssignmentKind::FunctionCall
+                        }
+                    };
+
+                    (
+                        kind,
+                        TypedExpr::Call {
+                            location,
+                            args,
+                            type_: return_type,
+                            fun: Box::new(func),
+                        },
+                    )
                 }
 
                 // left |> right(..args)
+                //         ^^^^^ This is `fun`
                 UntypedExpr::Call {
                     fun,
                     arguments,
                     location,
                     ..
                 } => {
-                    let fun = self.expr_typer.infer(*fun)?;
+                    let fun = match self.expr_typer.infer(*fun) {
+                        Ok(fun) => fun,
+                        Err(e) => {
+                            // In case we cannot infer the function we'll
+                            // replace it with an invalid expression with an
+                            // unbound type to keep going!
+                            self.expr_typer.problems.error(e);
+                            TypedExpr::Invalid {
+                                location,
+                                type_: self.expr_typer.new_unbound_var(),
+                            }
+                        }
+                    };
+
                     match fun.type_().fn_types() {
                         // Rewrite as right(..args)(left)
-                        Some((args, _)) if args.len() == arguments.len() => {
-                            self.infer_apply_to_call_pipe(fun, arguments, location)
-                        }
+                        Some((args, _)) if args.len() == arguments.len() => (
+                            PipelineAssignmentKind::FunctionCall,
+                            self.infer_apply_to_call_pipe(fun, arguments, location),
+                        ),
+
                         // Rewrite as right(left, ..args)
-                        _ => self.infer_insert_pipe(fun, arguments, location),
+                        _ => (
+                            PipelineAssignmentKind::FirstArgument {
+                                second_argument: arguments.first().map(|arg| arg.location),
+                            },
+                            self.infer_insert_pipe(fun, arguments, location),
+                        ),
                     }
                 }
 
+                UntypedExpr::Echo {
+                    location,
+                    expression: None,
+                } => {
+                    self.expr_typer.environment.echo_found = true;
+                    // An echo that is not followed by an expression that is
+                    // used as a pipeline's step is just like the identity
+                    // function.
+                    // So it gets the type of the value coming from the previous
+                    // step of the pipeline.
+                    (
+                        PipelineAssignmentKind::Echo,
+                        TypedExpr::Echo {
+                            location,
+                            expression: None,
+                            type_: self.argument_type.clone(),
+                        },
+                    )
+                }
+
                 // right(left)
-                call => self.infer_apply_pipe(call)?,
+                call => (
+                    PipelineAssignmentKind::FunctionCall,
+                    self.infer_apply_pipe(call),
+                ),
             };
 
             if i + 2 == self.size {
-                finally = Some(call);
+                finally = Some((call, kind));
             } else {
-                self.push_assignment(call);
+                self.push_assignment(call, kind);
             }
         }
 
-        Ok(finally.expect("Empty pipeline in typer"))
+        finally.expect("Empty pipeline in typer")
     }
 
     /// Create a call argument that can be used to refer to the value on the
@@ -175,6 +242,7 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
                 type_: self.argument_type.clone(),
                 variant: ValueConstructorVariant::LocalVariable {
                     location: self.argument_location,
+                    origin: VariableOrigin::Generated,
                 },
             },
         }
@@ -190,34 +258,11 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
     }
 
     /// Push an assignment for the value on the left hand side of the pipe
-    fn push_assignment(&mut self, expression: TypedExpr) {
+    fn push_assignment(&mut self, expression: TypedExpr, kind: PipelineAssignmentKind) {
         self.argument_type = expression.type_();
         self.argument_location = expression.location();
-        self.push_assignment_no_update(expression)
-    }
-
-    fn push_assignment_no_update(&mut self, expression: TypedExpr) {
-        let location = expression.location();
-        // Insert the variable for use in type checking the rest of the pipeline
-        self.expr_typer.environment.insert_local_variable(
-            PIPE_VARIABLE.into(),
-            location,
-            expression.type_(),
-        );
-        // Add the assignment to the AST
-        let assignment = Assignment {
-            location,
-            annotation: None,
-            kind: AssignmentKind::Generated,
-            pattern: Pattern::Variable {
-                location,
-                name: PIPE_VARIABLE.into(),
-                type_: expression.type_(),
-                origin: VariableOrigin::Generated,
-            },
-            value: Box::new(expression),
-        };
-        self.assignments.push(assignment);
+        let assignment = new_pipeline_assignment(self.expr_typer, expression);
+        self.assignments.push((assignment, kind));
     }
 
     /// Attempt to infer a |> b(..c) as b(..c)(a)
@@ -287,28 +332,47 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
     }
 
     /// Attempt to infer a |> b as b(a)
-    fn infer_apply_pipe(&mut self, function: UntypedExpr) -> Result<TypedExpr, Error> {
-        let function = Box::new(self.expr_typer.infer(function)?);
+    /// b is the `function` argument.
+    fn infer_apply_pipe(&mut self, function: UntypedExpr) -> TypedExpr {
+        let function_location = function.location();
+        let function = Box::new(match self.expr_typer.infer(function) {
+            Ok(function) => function,
+            Err(error) => {
+                // If we cannot infer the function we put an invalid expression
+                // in its place so we can still keep going with the other steps.
+                self.expr_typer.problems.error(error);
+                TypedExpr::Invalid {
+                    location: function_location,
+                    type_: self.expr_typer.new_unbound_var(),
+                }
+            }
+        });
+
         let return_type = self.expr_typer.new_unbound_var();
         // Ensure that the function accepts one argument of the correct type
-        unify(
+        let unification_result = unify(
             function.type_(),
             fn_(vec![self.argument_type.clone()], return_type.clone()),
-        )
-        .map_err(|e| {
-            if self.check_if_pipe_type_mismatch(&e) {
-                return convert_unify_error(e, function.location())
-                    .with_unify_error_situation(UnifyErrorSituation::PipeTypeMismatch);
+        );
+        match unification_result {
+            Ok(_) => (),
+            Err(error) => {
+                let error = if self.check_if_pipe_type_mismatch(&error) {
+                    convert_unify_error(error, function.location())
+                        .with_unify_error_situation(UnifyErrorSituation::PipeTypeMismatch)
+                } else {
+                    convert_unify_error(flip_unify_error(error), function.location())
+                };
+                self.expr_typer.problems.error(error);
             }
-            convert_unify_error(flip_unify_error(e), function.location())
-        })?;
+        };
 
-        Ok(TypedExpr::Call {
-            location: function.location(),
+        TypedExpr::Call {
+            location: function_location,
             type_: return_type,
             fun: function,
             args: vec![self.typed_left_hand_value_variable_call_argument()],
-        })
+        }
     }
 
     fn check_if_pipe_type_mismatch(&mut self, error: &UnifyError) -> bool {
@@ -354,3 +418,22 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
         }
     }
 }
+
+fn new_pipeline_assignment(
+    expr_typer: &mut ExprTyper<'_, '_>,
+    expression: TypedExpr,
+) -> TypedPipelineAssignment {
+    let location = expression.location();
+    // Insert the variable for use in type checking the rest of the pipeline
+    expr_typer.environment.insert_local_variable(
+        PIPE_VARIABLE.into(),
+        location,
+        VariableOrigin::Generated,
+        expression.type_(),
+    );
+    TypedPipelineAssignment {
+        location,
+        name: PIPE_VARIABLE.into(),
+        value: Box::new(expression),
+    }
+}
