@@ -1,9 +1,9 @@
 use bimap::BiMap;
-use ecow::EcoString;
+use ecow::{EcoString, eco_format};
 use im::HashMap;
 use std::{collections::HashSet, sync::Arc};
 
-use crate::type_::{collapse_links, Type, TypeVar};
+use crate::type_::{Type, TypeVar};
 
 /// This class keeps track of what names are used for modules in the current
 /// scope, so they can be printed in errors, etc.
@@ -126,9 +126,7 @@ fn compare_arguments(arguments: &[Arc<Type>], parameters: &[Arc<Type>]) -> bool
     arguments
         .iter()
         .zip(parameters)
-        .all(|(argument, parameter)| {
-            collapse_links(argument.clone()) == collapse_links(parameter.clone())
-        })
+        .all(|(argument, parameter)| argument.same_as(parameter))
 }
 
 impl Names {
@@ -323,10 +321,9 @@ impl<'a> Printer<'a> {
     }
 
     pub fn print_module(&self, module: &str) -> EcoString {
-        if let Some(module) = self.names.imported_modules.get(module) {
-            module.clone()
-        } else {
-            module.split("/").last().unwrap_or(module).into()
+        match self.names.imported_modules.get(module) {
+            Some(module) => module.clone(),
+            _ => module.split("/").last().unwrap_or(module).into(),
         }
     }
 
@@ -386,6 +383,21 @@ impl<'a> Printer<'a> {
         }
     }
 
+    pub fn print_constructor(&mut self, module: &EcoString, name: &EcoString) -> EcoString {
+        let (module, name) = match self.names.named_constructor(module, name) {
+            NameContextInformation::Qualified(module, name) => (Some(module), name),
+            NameContextInformation::Unqualified(name) => (None, name),
+            NameContextInformation::Unimported(name) => {
+                (Some(module.split('/').last().unwrap_or(module)), name)
+            }
+        };
+
+        match module {
+            Some(module) => eco_format!("{module}.{name}"),
+            None => name.into(),
+        }
+    }
+
     fn print_arguments(
         &mut self,
         args: &[Arc<Type>],
