@@ -22,6 +22,7 @@ mod assert;
 mod assignments;
 mod conditional_compilation;
 mod custom_types;
+mod echo;
 mod errors;
 mod exhaustiveness;
 mod externals;
@@ -231,7 +232,7 @@ fn get_warnings(
     warnings.take().into_iter().collect_vec()
 }
 
-fn get_printed_warnings(
+pub(crate) fn get_printed_warnings(
     src: &str,
     deps: Vec<DependencyModule<'_>>,
     target: Target,
@@ -346,6 +347,34 @@ macro_rules! assert_warnings_with_gleam_version {
     };
 }
 
+#[macro_export]
+macro_rules! assert_js_warnings_with_gleam_version {
+    ($gleam_version:expr, $src:expr$(,)?) => {
+        let warning = $crate::type_::tests::get_printed_warnings(
+            $src,
+            vec![],
+            crate::build::Target::JavaScript,
+            Some($gleam_version),
+        );
+        assert!(!warning.is_empty());
+        let output = format!("----- SOURCE CODE\n{}\n\n----- WARNING\n{}", $src, warning);
+        insta::assert_snapshot!(insta::internals::AutoName, output, $src);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_js_no_warnings_with_gleam_version {
+    ($gleam_version:expr, $src:expr$(,)?) => {
+        let warning = $crate::type_::tests::get_printed_warnings(
+            $src,
+            vec![],
+            crate::build::Target::JavaScript,
+            Some($gleam_version),
+        );
+        assert!(warning.is_empty());
+    };
+}
+
 #[macro_export]
 macro_rules! assert_no_warnings {
     ($src:expr $(,)?) => {
@@ -790,7 +819,6 @@ fn infer_module_type_retention_test() {
             package: "thepackage".into(),
             name: "ok".into(),
             is_internal: false,
-            // Core type constructors like String and Int are not included
             types: HashMap::new(),
             types_value_constructors: HashMap::from([
                 (
@@ -801,12 +829,15 @@ fn infer_module_type_retention_test() {
                             TypeValueConstructor {
                                 name: "True".into(),
                                 parameters: vec![],
+                                documentation: None,
                             },
                             TypeValueConstructor {
                                 name: "False".into(),
                                 parameters: vec![],
+                                documentation: None,
                             }
-                        ]
+                        ],
+                        opaque: Opaque::NotOpaque,
                     }
                 ),
                 (
@@ -818,15 +849,20 @@ fn infer_module_type_retention_test() {
                                 name: "Ok".into(),
                                 parameters: vec![TypeValueConstructorField {
                                     type_: generic_var(1),
-                                }]
+                                    label: None,
+                                }],
+                                documentation: None,
                             },
                             TypeValueConstructor {
                                 name: "Error".into(),
                                 parameters: vec![TypeValueConstructorField {
                                     type_: generic_var(2),
-                                }]
+                                    label: None,
+                                }],
+                                documentation: None,
                             }
-                        ]
+                        ],
+                        opaque: Opaque::NotOpaque,
                     }
                 ),
                 (
@@ -835,8 +871,10 @@ fn infer_module_type_retention_test() {
                         type_parameters_ids: vec![],
                         variants: vec![TypeValueConstructor {
                             name: "Nil".into(),
-                            parameters: vec![]
-                        }]
+                            parameters: vec![],
+                            documentation: None,
+                        }],
+                        opaque: Opaque::NotOpaque,
                     }
                 )
             ]),
@@ -845,6 +883,9 @@ fn infer_module_type_retention_test() {
             line_numbers: LineNumbers::new(""),
             src_path: "".into(),
             minimum_required_version: Version::new(0, 1, 0),
+            type_aliases: HashMap::new(),
+            documentation: Vec::new(),
+            contains_echo: false,
         }
     );
 }
