@@ -2577,8 +2577,8 @@ pub fn main() {
     );
 }
 
-#[test]
 // https://github.com/gleam-lang/gleam/issues/3783
+#[test]
 fn duplicate_fields_in_record_update_reports_error() {
     assert_module_error!(
         "
@@ -2655,3 +2655,319 @@ todo
 "#
     );
 }
+
+#[test]
+fn out_of_range_erlang_float() {
+    assert_error!(r#"1.8e308"#);
+}
+
+#[test]
+fn out_of_range_erlang_float_in_pattern() {
+    assert_error!(r#"let assert [1.8e308, b] = [x, y]"#);
+}
+
+#[test]
+fn out_of_range_erlang_float_in_const() {
+    assert_module_error!(r#"const x = 1.8e308"#);
+}
+
+#[test]
+fn negative_out_of_range_erlang_float() {
+    assert_error!(r#"-1.8e308"#);
+}
+
+#[test]
+fn negative_out_of_range_erlang_float_in_pattern() {
+    assert_error!(r#"let assert [-1.8e308, b] = [x, y]"#);
+}
+
+#[test]
+fn negative_out_of_range_erlang_float_in_const() {
+    assert_module_error!(r#"const x = -1.8e308"#);
+}
+
+#[test]
+fn missing_case_body() {
+    assert_error!("case True");
+}
+
+#[test]
+fn suggest_wrapping_a_value_into_ok_if_types_match() {
+    assert_module_error!(
+        "
+pub fn main() {
+  case todo {
+    1 -> Ok(2)
+    _ -> 1
+  }
+}
+"
+    );
+}
+
+#[test]
+fn suggest_wrapping_a_value_into_ok_if_types_match_2() {
+    assert_module_error!(
+        "
+pub fn main() {
+  wibble(1)
+}
+
+fn wibble(arg: Result(Int, String)) { todo }
+"
+    );
+}
+
+#[test]
+fn suggest_wrapping_a_value_into_ok_if_types_match_with_block() {
+    assert_module_error!(
+        "
+pub fn main() {
+  case todo {
+    1 -> Ok(2)
+    _ -> {
+      todo
+      1
+    }
+  }
+}
+"
+    );
+}
+
+#[test]
+fn suggest_wrapping_a_value_into_ok_with_generic_type() {
+    assert_module_error!(
+        "
+pub fn first(list: List(a)) -> Result(a, Nil) {
+  case list {
+    [] -> Error(Nil)
+    [first, ..rest] -> first
+  }
+}
+"
+    );
+}
+
+#[test]
+fn suggest_wrapping_a_value_into_ok_if_types_match_with_multiline_result_in_block() {
+    assert_module_error!(
+        "
+pub fn main() {
+  case todo {
+    1 -> Ok(2)
+    _ -> {
+      todo
+      1
+      |> add_1
+    }
+  }
+}
+
+fn add_1(n: Int) { n + 1 }
+"
+    );
+}
+
+#[test]
+fn suggest_wrapping_a_value_into_error_if_types_match() {
+    assert_module_error!(
+        "
+pub fn main() {
+  case todo {
+    1 -> Error(1)
+    _ -> 1
+  }
+}
+"
+    );
+}
+
+#[test]
+fn suggest_wrapping_a_value_into_error_if_types_match_2() {
+    assert_module_error!(
+        "
+pub fn main() {
+    wibble(\"a\")
+}
+
+fn wibble(arg: Result(Int, String)) { todo }
+"
+    );
+}
+
+#[test]
+fn suggest_wrapping_a_function_return_value_in_ok() {
+    assert_module_error!(
+        "
+pub fn main() -> Result(Int, Bool) {
+  1
+}
+"
+    );
+}
+
+#[test]
+fn suggest_wrapping_a_function_return_value_in_error() {
+    assert_module_error!(
+        "
+pub fn main() -> Result(Int, Bool) {
+  True
+}
+"
+    );
+}
+
+#[test]
+fn suggest_wrapping_a_use_returned_value_in_ok() {
+    assert_module_error!(
+        "
+pub fn main() -> Result(Int, Bool) {
+  use <- want_result
+  1
+}
+
+pub fn want_result(wibble: fn() -> Result(Int, Bool)) {
+  todo
+}
+"
+    );
+}
+
+#[test]
+fn suggest_wrapping_a_use_returned_value_in_error() {
+    assert_module_error!(
+        "
+pub fn main() -> Result(Int, Bool) {
+  use <- want_result
+  False
+}
+
+pub fn want_result(wibble: fn() -> Result(Int, Bool)) {
+  todo
+}
+"
+    );
+}
+
+#[test]
+// https://github.com/gleam-lang/gleam/issues/4195
+fn let_assert_binding_cannot_be_used_in_panic_message() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  let assert Ok(message) = Error("Not Message") as { "Uh oh: " <> message }
+}
+"#
+    );
+}
+
+#[test]
+fn echo_followed_by_no_expression() {
+    assert_error!("echo");
+}
+
+#[test]
+fn echo_followed_by_no_expression_2() {
+    assert_module_error!(
+        r#"
+  pub fn wibble(a) { a }
+
+  pub fn main() {
+    wibble(echo)
+  }
+"#
+    );
+}
+
+#[test]
+fn echo_followed_by_no_expression_3() {
+    assert_module_error!(
+        r#"
+  pub fn main() {
+    echo + 1
+  }
+"#
+    );
+}
+
+#[test]
+fn echo_followed_by_no_expression_4() {
+    assert_module_error!(
+        r#"
+  pub fn main() {
+    "wibble" <> echo
+  }
+"#
+    );
+}
+
+#[test]
+fn echo_followed_by_no_expression_5() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  panic as echo
+}
+"#
+    );
+}
+
+#[test]
+fn echo_followed_by_no_expression_6() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  [echo, 1, 2]
+}
+"#
+    );
+}
+#[test]
+fn echo_followed_by_no_expression_7() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  #(1, echo)
+}
+"#
+    );
+}
+
+#[test]
+fn echo_followed_by_no_expression_8() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  todo
+  |> fn(_) { echo }
+  |> todo
+}
+"#
+    );
+}
+
+#[test]
+fn echo_followed_by_no_expression_9() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  todo
+  |> { echo }
+  |> todo
+}
+"#
+    );
+}
+
+#[test]
+fn echo_followed_by_no_expression_10() {
+    assert_module_error!(
+        r#"
+pub fn main() {
+  echo
+  |> todo
+}
+"#
+    );
+}
