@@ -2989,3 +2989,28 @@ pub fn main() {
 "#
     );
 }
+
+#[test]
+fn echo_followed_by_panic() {
+    assert_warning!(
+        "
+pub fn main() {
+  echo panic
+}
+"
+    );
+}
+
+#[test]
+fn echo_followed_by_panicking_expression() {
+    assert_warning!(
+        "
+pub fn main(a) {
+  echo case a {
+    1 -> panic
+    _ -> [1, panic]
+  }
+}
+"
+    );
+}
